YNN_ELEMENTWISE_KERNEL(arch_flag::neon, abs_fp32_1x8_neon, abs, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, ceil_fp32_1x8_neon, ceil, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, cosine_fp32_1x8_neon, cosine, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, erf_fp32_1x8_neon, erf, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, exp_fp32_1x8_neon, exp, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, floor_fp32_1x8_neon, floor, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, negate_fp32_1x8_neon, negate, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, reciprocal_square_root_fp32_1x8_neon, reciprocal_square_root, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, round_fp32_1x8_neon, round, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, sine_fp32_1x8_neon, sine, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, square_root_fp32_1x8_neon, square_root, nullptr, float32x1_t, float32x1_t)
YNN_ELEMENTWISE_KERNEL(arch_flag::neon, tanh_fp32_1x8_neon, tanh, nullptr, float32x1_t, float32x1_t)
