YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_1x16x2_1x4x2_avx2_fma3, 1, 16, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_2x16x2_1x4x2_avx2_fma3, 2, 16, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_3x8x2_1x4x2_avx2_fma3, 3, 8, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_4x8x2_1x4x2_avx2_fma3, 4, 8, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_5x8x2_1x4x2_avx2_fma3, 5, 8, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_6x8x2_1x4x2_avx2_fma3, 6, 8, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_4x4x2_1x4x2_avx2_fma3, 4, 4, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_5x4x2_1x4x2_avx2_fma3, 5, 4, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_6x4x2_1x4x2_avx2_fma3, 6, 4, 2, 1, 4, 2, /*flags=*/0, float, float, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_fp32_8x4x2_1x4x2_avx2_fma3, 8, 4, 2, 1, 4, 2, /*flags=*/0, float, float, float)
