
/* Autogenerated by mlir-tblgen; don't manually edit. */

#include "mlir-c/Pass.h"

#ifdef __cplusplus
extern "C" {
#endif

// Registration for the entire group
MLIR_CAPI_EXPORTED void mlirRegisterPasses(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateArithEmulateUnsupportedFloats(void);
MLIR_CAPI_EXPORTED void mlirRegisterArithEmulateUnsupportedFloats(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateArithEmulateWideInt(void);
MLIR_CAPI_EXPORTED void mlirRegisterArithEmulateWideInt(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateArithExpandOpsPass(void);
MLIR_CAPI_EXPORTED void mlirRegisterArithExpandOpsPass(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateArithIntRangeNarrowing(void);
MLIR_CAPI_EXPORTED void mlirRegisterArithIntRangeNarrowing(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateArithIntRangeOpts(void);
MLIR_CAPI_EXPORTED void mlirRegisterArithIntRangeOpts(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateArithUnsignedWhenEquivalentPass(void);
MLIR_CAPI_EXPORTED void mlirRegisterArithUnsignedWhenEquivalentPass(void);



#ifdef __cplusplus
}
#endif
