/* Copyright 2025 The OpenXLA Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/

#ifndef XLA_PJRT_PLUGIN_XLA_CPU_CPU_MEMORY_H_
#define XLA_PJRT_PLUGIN_XLA_CPU_CPU_MEMORY_H_

#include <cstddef>

namespace xla {

// A region of cpu memory that can be used by XLA:CPU PjRt client as a storage
// for PjRt buffers and also as a temporary allocation for XLA:CPU program.
class CpuMemory {
 public:
  virtual ~CpuMemory() = default;

  virtual void* base() const = 0;
  virtual size_t size_bytes() const = 0;
};

}  // namespace xla

#endif  // XLA_PJRT_PLUGIN_XLA_CPU_CPU_MEMORY_H_
