// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: xla/stream_executor/kernel_args_packing_spec.proto
// Protobuf C++ Version: 6.31.1

#ifndef xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto_2epb_2eh
#define xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6031001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable descriptor_table_xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto;
}  // extern "C"
namespace stream_executor {
enum KernelArgPackingRelocationProto_Kind : int;
PROTOBUF_EXPORT extern const uint32_t KernelArgPackingRelocationProto_Kind_internal_data_[];
class KernelArgPackingRelocationProto;
struct KernelArgPackingRelocationProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern KernelArgPackingRelocationProtoDefaultTypeInternal _KernelArgPackingRelocationProto_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull KernelArgPackingRelocationProto_class_data_;
class KernelArgPackingSpecProto;
struct KernelArgPackingSpecProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern KernelArgPackingSpecProtoDefaultTypeInternal _KernelArgPackingSpecProto_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull KernelArgPackingSpecProto_class_data_;
class KernelArgsPackingSpecProto;
struct KernelArgsPackingSpecProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern KernelArgsPackingSpecProtoDefaultTypeInternal _KernelArgsPackingSpecProto_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull KernelArgsPackingSpecProto_class_data_;
}  // namespace stream_executor
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::stream_executor::KernelArgPackingRelocationProto_Kind_internal_data_>
    internal::EnumTraitsImpl::value<::stream_executor::KernelArgPackingRelocationProto_Kind>;
}  // namespace protobuf
}  // namespace google

namespace stream_executor {
enum KernelArgPackingRelocationProto_Kind : int {
  KernelArgPackingRelocationProto_Kind_KIND_UNSPECIFIED = 0,
  KernelArgPackingRelocationProto_Kind_KIND_BITS64_ABSOLUTE = 1,
  KernelArgPackingRelocationProto_Kind_KernelArgPackingRelocationProto_Kind_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  KernelArgPackingRelocationProto_Kind_KernelArgPackingRelocationProto_Kind_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

PROTOBUF_EXPORT extern const uint32_t KernelArgPackingRelocationProto_Kind_internal_data_[];
inline constexpr KernelArgPackingRelocationProto_Kind KernelArgPackingRelocationProto_Kind_Kind_MIN =
    static_cast<KernelArgPackingRelocationProto_Kind>(0);
inline constexpr KernelArgPackingRelocationProto_Kind KernelArgPackingRelocationProto_Kind_Kind_MAX =
    static_cast<KernelArgPackingRelocationProto_Kind>(1);
inline bool KernelArgPackingRelocationProto_Kind_IsValid(int value) {
  return 0 <= value && value <= 1;
}
inline constexpr int KernelArgPackingRelocationProto_Kind_Kind_ARRAYSIZE = 1 + 1;
PROTOBUF_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL KernelArgPackingRelocationProto_Kind_descriptor();
template <typename T>
const ::std::string& KernelArgPackingRelocationProto_Kind_Name(T value) {
  static_assert(::std::is_same<T, KernelArgPackingRelocationProto_Kind>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Kind_Name().");
  return KernelArgPackingRelocationProto_Kind_Name(static_cast<KernelArgPackingRelocationProto_Kind>(value));
}
template <>
inline const ::std::string& KernelArgPackingRelocationProto_Kind_Name(KernelArgPackingRelocationProto_Kind value) {
  return ::google::protobuf::internal::NameOfDenseEnum<KernelArgPackingRelocationProto_Kind_descriptor, 0, 1>(
      static_cast<int>(value));
}
inline bool KernelArgPackingRelocationProto_Kind_Parse(
    ::absl::string_view name, KernelArgPackingRelocationProto_Kind* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<KernelArgPackingRelocationProto_Kind>(KernelArgPackingRelocationProto_Kind_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT KernelArgPackingRelocationProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:stream_executor.KernelArgPackingRelocationProto) */ {
 public:
  inline KernelArgPackingRelocationProto() : KernelArgPackingRelocationProto(nullptr) {}
  ~KernelArgPackingRelocationProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(KernelArgPackingRelocationProto* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(KernelArgPackingRelocationProto));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR KernelArgPackingRelocationProto(::google::protobuf::internal::ConstantInitialized);

  inline KernelArgPackingRelocationProto(const KernelArgPackingRelocationProto& from) : KernelArgPackingRelocationProto(nullptr, from) {}
  inline KernelArgPackingRelocationProto(KernelArgPackingRelocationProto&& from) noexcept
      : KernelArgPackingRelocationProto(nullptr, ::std::move(from)) {}
  inline KernelArgPackingRelocationProto& operator=(const KernelArgPackingRelocationProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline KernelArgPackingRelocationProto& operator=(KernelArgPackingRelocationProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const KernelArgPackingRelocationProto& default_instance() {
    return *reinterpret_cast<const KernelArgPackingRelocationProto*>(
        &_KernelArgPackingRelocationProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(KernelArgPackingRelocationProto& a, KernelArgPackingRelocationProto& b) { a.Swap(&b); }
  inline void Swap(KernelArgPackingRelocationProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(KernelArgPackingRelocationProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  KernelArgPackingRelocationProto* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<KernelArgPackingRelocationProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const KernelArgPackingRelocationProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const KernelArgPackingRelocationProto& from) { KernelArgPackingRelocationProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(KernelArgPackingRelocationProto* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "stream_executor.KernelArgPackingRelocationProto"; }

 protected:
  explicit KernelArgPackingRelocationProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  KernelArgPackingRelocationProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const KernelArgPackingRelocationProto& from);
  KernelArgPackingRelocationProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, KernelArgPackingRelocationProto&& from) noexcept
      : KernelArgPackingRelocationProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Kind = KernelArgPackingRelocationProto_Kind;
  static constexpr Kind KIND_UNSPECIFIED = KernelArgPackingRelocationProto_Kind_KIND_UNSPECIFIED;
  static constexpr Kind KIND_BITS64_ABSOLUTE = KernelArgPackingRelocationProto_Kind_KIND_BITS64_ABSOLUTE;
  static inline bool Kind_IsValid(int value) {
    return KernelArgPackingRelocationProto_Kind_IsValid(value);
  }
  static constexpr Kind Kind_MIN = KernelArgPackingRelocationProto_Kind_Kind_MIN;
  static constexpr Kind Kind_MAX = KernelArgPackingRelocationProto_Kind_Kind_MAX;
  static constexpr int Kind_ARRAYSIZE = KernelArgPackingRelocationProto_Kind_Kind_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Kind_descriptor() {
    return KernelArgPackingRelocationProto_Kind_descriptor();
  }
  template <typename T>
  static inline const ::std::string& Kind_Name(T value) {
    return KernelArgPackingRelocationProto_Kind_Name(value);
  }
  static inline bool Kind_Parse(
      ::absl::string_view name, Kind* PROTOBUF_NONNULL value) {
    return KernelArgPackingRelocationProto_Kind_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kKindFieldNumber = 1,
    kArgumentIndexFieldNumber = 2,
    kOffsetFieldNumber = 3,
  };
  // .stream_executor.KernelArgPackingRelocationProto.Kind kind = 1;
  void clear_kind() ;
  ::stream_executor::KernelArgPackingRelocationProto_Kind kind() const;
  void set_kind(::stream_executor::KernelArgPackingRelocationProto_Kind value);

  private:
  ::stream_executor::KernelArgPackingRelocationProto_Kind _internal_kind() const;
  void _internal_set_kind(::stream_executor::KernelArgPackingRelocationProto_Kind value);

  public:
  // int32 argument_index = 2;
  void clear_argument_index() ;
  ::int32_t argument_index() const;
  void set_argument_index(::int32_t value);

  private:
  ::int32_t _internal_argument_index() const;
  void _internal_set_argument_index(::int32_t value);

  public:
  // int64 offset = 3;
  void clear_offset() ;
  ::int64_t offset() const;
  void set_offset(::int64_t value);

  private:
  ::int64_t _internal_offset() const;
  void _internal_set_offset(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:stream_executor.KernelArgPackingRelocationProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const KernelArgPackingRelocationProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    int kind_;
    ::int32_t argument_index_;
    ::int64_t offset_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull KernelArgPackingRelocationProto_class_data_;
// -------------------------------------------------------------------

class PROTOBUF_EXPORT KernelArgPackingSpecProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:stream_executor.KernelArgPackingSpecProto) */ {
 public:
  inline KernelArgPackingSpecProto() : KernelArgPackingSpecProto(nullptr) {}
  ~KernelArgPackingSpecProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(KernelArgPackingSpecProto* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(KernelArgPackingSpecProto));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR KernelArgPackingSpecProto(::google::protobuf::internal::ConstantInitialized);

  inline KernelArgPackingSpecProto(const KernelArgPackingSpecProto& from) : KernelArgPackingSpecProto(nullptr, from) {}
  inline KernelArgPackingSpecProto(KernelArgPackingSpecProto&& from) noexcept
      : KernelArgPackingSpecProto(nullptr, ::std::move(from)) {}
  inline KernelArgPackingSpecProto& operator=(const KernelArgPackingSpecProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline KernelArgPackingSpecProto& operator=(KernelArgPackingSpecProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const KernelArgPackingSpecProto& default_instance() {
    return *reinterpret_cast<const KernelArgPackingSpecProto*>(
        &_KernelArgPackingSpecProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(KernelArgPackingSpecProto& a, KernelArgPackingSpecProto& b) { a.Swap(&b); }
  inline void Swap(KernelArgPackingSpecProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(KernelArgPackingSpecProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  KernelArgPackingSpecProto* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<KernelArgPackingSpecProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const KernelArgPackingSpecProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const KernelArgPackingSpecProto& from) { KernelArgPackingSpecProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(KernelArgPackingSpecProto* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "stream_executor.KernelArgPackingSpecProto"; }

 protected:
  explicit KernelArgPackingSpecProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  KernelArgPackingSpecProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const KernelArgPackingSpecProto& from);
  KernelArgPackingSpecProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, KernelArgPackingSpecProto&& from) noexcept
      : KernelArgPackingSpecProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kRelocationsFieldNumber = 1,
    kDataFieldNumber = 2,
  };
  // repeated .stream_executor.KernelArgPackingRelocationProto relocations = 1;
  int relocations_size() const;
  private:
  int _internal_relocations_size() const;

  public:
  void clear_relocations() ;
  ::stream_executor::KernelArgPackingRelocationProto* PROTOBUF_NONNULL mutable_relocations(int index);
  ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingRelocationProto>* PROTOBUF_NONNULL mutable_relocations();

  private:
  const ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingRelocationProto>& _internal_relocations() const;
  ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingRelocationProto>* PROTOBUF_NONNULL _internal_mutable_relocations();
  public:
  const ::stream_executor::KernelArgPackingRelocationProto& relocations(int index) const;
  ::stream_executor::KernelArgPackingRelocationProto* PROTOBUF_NONNULL add_relocations();
  const ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingRelocationProto>& relocations() const;
  // bytes data = 2;
  void clear_data() ;
  const ::std::string& data() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_data(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_data();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_data();
  void set_allocated_data(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_data() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_data(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_data();

  public:
  // @@protoc_insertion_point(class_scope:stream_executor.KernelArgPackingSpecProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const KernelArgPackingSpecProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::stream_executor::KernelArgPackingRelocationProto > relocations_;
    ::google::protobuf::internal::ArenaStringPtr data_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull KernelArgPackingSpecProto_class_data_;
// -------------------------------------------------------------------

class PROTOBUF_EXPORT KernelArgsPackingSpecProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:stream_executor.KernelArgsPackingSpecProto) */ {
 public:
  inline KernelArgsPackingSpecProto() : KernelArgsPackingSpecProto(nullptr) {}
  ~KernelArgsPackingSpecProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(KernelArgsPackingSpecProto* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(KernelArgsPackingSpecProto));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR KernelArgsPackingSpecProto(::google::protobuf::internal::ConstantInitialized);

  inline KernelArgsPackingSpecProto(const KernelArgsPackingSpecProto& from) : KernelArgsPackingSpecProto(nullptr, from) {}
  inline KernelArgsPackingSpecProto(KernelArgsPackingSpecProto&& from) noexcept
      : KernelArgsPackingSpecProto(nullptr, ::std::move(from)) {}
  inline KernelArgsPackingSpecProto& operator=(const KernelArgsPackingSpecProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline KernelArgsPackingSpecProto& operator=(KernelArgsPackingSpecProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const KernelArgsPackingSpecProto& default_instance() {
    return *reinterpret_cast<const KernelArgsPackingSpecProto*>(
        &_KernelArgsPackingSpecProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(KernelArgsPackingSpecProto& a, KernelArgsPackingSpecProto& b) { a.Swap(&b); }
  inline void Swap(KernelArgsPackingSpecProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(KernelArgsPackingSpecProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  KernelArgsPackingSpecProto* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<KernelArgsPackingSpecProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const KernelArgsPackingSpecProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const KernelArgsPackingSpecProto& from) { KernelArgsPackingSpecProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(KernelArgsPackingSpecProto* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "stream_executor.KernelArgsPackingSpecProto"; }

 protected:
  explicit KernelArgsPackingSpecProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  KernelArgsPackingSpecProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const KernelArgsPackingSpecProto& from);
  KernelArgsPackingSpecProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, KernelArgsPackingSpecProto&& from) noexcept
      : KernelArgsPackingSpecProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kKernelArgumentsFieldNumber = 1,
  };
  // repeated .stream_executor.KernelArgPackingSpecProto kernel_arguments = 1;
  int kernel_arguments_size() const;
  private:
  int _internal_kernel_arguments_size() const;

  public:
  void clear_kernel_arguments() ;
  ::stream_executor::KernelArgPackingSpecProto* PROTOBUF_NONNULL mutable_kernel_arguments(int index);
  ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingSpecProto>* PROTOBUF_NONNULL mutable_kernel_arguments();

  private:
  const ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingSpecProto>& _internal_kernel_arguments() const;
  ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingSpecProto>* PROTOBUF_NONNULL _internal_mutable_kernel_arguments();
  public:
  const ::stream_executor::KernelArgPackingSpecProto& kernel_arguments(int index) const;
  ::stream_executor::KernelArgPackingSpecProto* PROTOBUF_NONNULL add_kernel_arguments();
  const ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingSpecProto>& kernel_arguments() const;
  // @@protoc_insertion_point(class_scope:stream_executor.KernelArgsPackingSpecProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const KernelArgsPackingSpecProto& from_msg);
    ::google::protobuf::RepeatedPtrField< ::stream_executor::KernelArgPackingSpecProto > kernel_arguments_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull KernelArgsPackingSpecProto_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// KernelArgPackingRelocationProto

// .stream_executor.KernelArgPackingRelocationProto.Kind kind = 1;
inline void KernelArgPackingRelocationProto::clear_kind() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.kind_ = 0;
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline ::stream_executor::KernelArgPackingRelocationProto_Kind KernelArgPackingRelocationProto::kind() const {
  // @@protoc_insertion_point(field_get:stream_executor.KernelArgPackingRelocationProto.kind)
  return _internal_kind();
}
inline void KernelArgPackingRelocationProto::set_kind(::stream_executor::KernelArgPackingRelocationProto_Kind value) {
  _internal_set_kind(value);
  _impl_._has_bits_[0] |= 0x00000001u;
  // @@protoc_insertion_point(field_set:stream_executor.KernelArgPackingRelocationProto.kind)
}
inline ::stream_executor::KernelArgPackingRelocationProto_Kind KernelArgPackingRelocationProto::_internal_kind() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::stream_executor::KernelArgPackingRelocationProto_Kind>(_impl_.kind_);
}
inline void KernelArgPackingRelocationProto::_internal_set_kind(::stream_executor::KernelArgPackingRelocationProto_Kind value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.kind_ = value;
}

// int32 argument_index = 2;
inline void KernelArgPackingRelocationProto::clear_argument_index() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.argument_index_ = 0;
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline ::int32_t KernelArgPackingRelocationProto::argument_index() const {
  // @@protoc_insertion_point(field_get:stream_executor.KernelArgPackingRelocationProto.argument_index)
  return _internal_argument_index();
}
inline void KernelArgPackingRelocationProto::set_argument_index(::int32_t value) {
  _internal_set_argument_index(value);
  _impl_._has_bits_[0] |= 0x00000002u;
  // @@protoc_insertion_point(field_set:stream_executor.KernelArgPackingRelocationProto.argument_index)
}
inline ::int32_t KernelArgPackingRelocationProto::_internal_argument_index() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.argument_index_;
}
inline void KernelArgPackingRelocationProto::_internal_set_argument_index(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.argument_index_ = value;
}

// int64 offset = 3;
inline void KernelArgPackingRelocationProto::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline ::int64_t KernelArgPackingRelocationProto::offset() const {
  // @@protoc_insertion_point(field_get:stream_executor.KernelArgPackingRelocationProto.offset)
  return _internal_offset();
}
inline void KernelArgPackingRelocationProto::set_offset(::int64_t value) {
  _internal_set_offset(value);
  _impl_._has_bits_[0] |= 0x00000004u;
  // @@protoc_insertion_point(field_set:stream_executor.KernelArgPackingRelocationProto.offset)
}
inline ::int64_t KernelArgPackingRelocationProto::_internal_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.offset_;
}
inline void KernelArgPackingRelocationProto::_internal_set_offset(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = value;
}

// -------------------------------------------------------------------

// KernelArgPackingSpecProto

// repeated .stream_executor.KernelArgPackingRelocationProto relocations = 1;
inline int KernelArgPackingSpecProto::_internal_relocations_size() const {
  return _internal_relocations().size();
}
inline int KernelArgPackingSpecProto::relocations_size() const {
  return _internal_relocations_size();
}
inline void KernelArgPackingSpecProto::clear_relocations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.relocations_.Clear();
}
inline ::stream_executor::KernelArgPackingRelocationProto* PROTOBUF_NONNULL KernelArgPackingSpecProto::mutable_relocations(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:stream_executor.KernelArgPackingSpecProto.relocations)
  return _internal_mutable_relocations()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingRelocationProto>* PROTOBUF_NONNULL KernelArgPackingSpecProto::mutable_relocations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:stream_executor.KernelArgPackingSpecProto.relocations)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_relocations();
}
inline const ::stream_executor::KernelArgPackingRelocationProto& KernelArgPackingSpecProto::relocations(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:stream_executor.KernelArgPackingSpecProto.relocations)
  return _internal_relocations().Get(index);
}
inline ::stream_executor::KernelArgPackingRelocationProto* PROTOBUF_NONNULL KernelArgPackingSpecProto::add_relocations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::stream_executor::KernelArgPackingRelocationProto* _add = _internal_mutable_relocations()->Add();
  // @@protoc_insertion_point(field_add:stream_executor.KernelArgPackingSpecProto.relocations)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingRelocationProto>& KernelArgPackingSpecProto::relocations() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:stream_executor.KernelArgPackingSpecProto.relocations)
  return _internal_relocations();
}
inline const ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingRelocationProto>&
KernelArgPackingSpecProto::_internal_relocations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.relocations_;
}
inline ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingRelocationProto>* PROTOBUF_NONNULL
KernelArgPackingSpecProto::_internal_mutable_relocations() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.relocations_;
}

// bytes data = 2;
inline void KernelArgPackingSpecProto::clear_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::std::string& KernelArgPackingSpecProto::data() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:stream_executor.KernelArgPackingSpecProto.data)
  return _internal_data();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void KernelArgPackingSpecProto::set_data(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.data_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:stream_executor.KernelArgPackingSpecProto.data)
}
inline ::std::string* PROTOBUF_NONNULL KernelArgPackingSpecProto::mutable_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_data();
  // @@protoc_insertion_point(field_mutable:stream_executor.KernelArgPackingSpecProto.data)
  return _s;
}
inline const ::std::string& KernelArgPackingSpecProto::_internal_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_.Get();
}
inline void KernelArgPackingSpecProto::_internal_set_data(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.data_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL KernelArgPackingSpecProto::_internal_mutable_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.data_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE KernelArgPackingSpecProto::release_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:stream_executor.KernelArgPackingSpecProto.data)
  if ((_impl_._has_bits_[0] & 0x00000001u) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* released = _impl_.data_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.data_.Set("", GetArena());
  }
  return released;
}
inline void KernelArgPackingSpecProto::set_allocated_data(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.data_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.data_.IsDefault()) {
    _impl_.data_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:stream_executor.KernelArgPackingSpecProto.data)
}

// -------------------------------------------------------------------

// KernelArgsPackingSpecProto

// repeated .stream_executor.KernelArgPackingSpecProto kernel_arguments = 1;
inline int KernelArgsPackingSpecProto::_internal_kernel_arguments_size() const {
  return _internal_kernel_arguments().size();
}
inline int KernelArgsPackingSpecProto::kernel_arguments_size() const {
  return _internal_kernel_arguments_size();
}
inline void KernelArgsPackingSpecProto::clear_kernel_arguments() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.kernel_arguments_.Clear();
}
inline ::stream_executor::KernelArgPackingSpecProto* PROTOBUF_NONNULL KernelArgsPackingSpecProto::mutable_kernel_arguments(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:stream_executor.KernelArgsPackingSpecProto.kernel_arguments)
  return _internal_mutable_kernel_arguments()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingSpecProto>* PROTOBUF_NONNULL KernelArgsPackingSpecProto::mutable_kernel_arguments()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:stream_executor.KernelArgsPackingSpecProto.kernel_arguments)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_kernel_arguments();
}
inline const ::stream_executor::KernelArgPackingSpecProto& KernelArgsPackingSpecProto::kernel_arguments(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:stream_executor.KernelArgsPackingSpecProto.kernel_arguments)
  return _internal_kernel_arguments().Get(index);
}
inline ::stream_executor::KernelArgPackingSpecProto* PROTOBUF_NONNULL KernelArgsPackingSpecProto::add_kernel_arguments()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::stream_executor::KernelArgPackingSpecProto* _add = _internal_mutable_kernel_arguments()->Add();
  // @@protoc_insertion_point(field_add:stream_executor.KernelArgsPackingSpecProto.kernel_arguments)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingSpecProto>& KernelArgsPackingSpecProto::kernel_arguments() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:stream_executor.KernelArgsPackingSpecProto.kernel_arguments)
  return _internal_kernel_arguments();
}
inline const ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingSpecProto>&
KernelArgsPackingSpecProto::_internal_kernel_arguments() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.kernel_arguments_;
}
inline ::google::protobuf::RepeatedPtrField<::stream_executor::KernelArgPackingSpecProto>* PROTOBUF_NONNULL
KernelArgsPackingSpecProto::_internal_mutable_kernel_arguments() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.kernel_arguments_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace stream_executor


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::stream_executor::KernelArgPackingRelocationProto_Kind> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::stream_executor::KernelArgPackingRelocationProto_Kind>() {
  return ::stream_executor::KernelArgPackingRelocationProto_Kind_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // xla_2fstream_5fexecutor_2fkernel_5fargs_5fpacking_5fspec_2eproto_2epb_2eh
