YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_1x64x2_1x16x2_avx512bf16, 1, 64, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_2x64x2_1x16x2_avx512bf16, 2, 64, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_3x64x2_1x16x2_avx512bf16, 3, 64, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_4x64x2_1x16x2_avx512bf16, 4, 64, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_5x64x2_1x16x2_avx512bf16, 5, 64, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_2x32x2_1x16x2_avx512bf16, 2, 32, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_3x32x2_1x16x2_avx512bf16, 3, 32, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_4x32x2_1x16x2_avx512bf16, 4, 32, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_5x32x2_1x16x2_avx512bf16, 5, 32, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_6x32x2_1x16x2_avx512bf16, 6, 32, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_8x32x2_1x16x2_avx512bf16, 8, 32, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_10x32x2_1x16x2_avx512bf16, 10, 32, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_12x32x2_1x16x2_avx512bf16, 12, 32, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx512bf16, dot_bf16_bf16_fp32_16x16x2_1x16x2_avx512bf16, 16, 16, 2, 1, 16, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
