// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: xla/stream_executor/gpu/gpu_blas_lt.proto
// Protobuf C++ Version: 6.31.1

#ifndef xla_2fstream_5fexecutor_2fgpu_2fgpu_5fblas_5flt_2eproto_2epb_2eh
#define xla_2fstream_5fexecutor_2fgpu_2fgpu_5fblas_5flt_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6031001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
#include "xla/stream_executor/blas.pb.h"
#include "xla/xla_data.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_xla_2fstream_5fexecutor_2fgpu_2fgpu_5fblas_5flt_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_xla_2fstream_5fexecutor_2fgpu_2fgpu_5fblas_5flt_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable descriptor_table_xla_2fstream_5fexecutor_2fgpu_2fgpu_5fblas_5flt_2eproto;
}  // extern "C"
namespace xla {
enum BlasLtEpilogueProto : int;
PROTOBUF_EXPORT extern const uint32_t BlasLtEpilogueProto_internal_data_[];
enum GemmConfigProto_MatrixLayout_Order : int;
PROTOBUF_EXPORT extern const uint32_t GemmConfigProto_MatrixLayout_Order_internal_data_[];
class GemmConfigProto;
struct GemmConfigProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern GemmConfigProtoDefaultTypeInternal _GemmConfigProto_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull GemmConfigProto_class_data_;
class GemmConfigProto_MatrixLayout;
struct GemmConfigProto_MatrixLayoutDefaultTypeInternal;
PROTOBUF_EXPORT extern GemmConfigProto_MatrixLayoutDefaultTypeInternal _GemmConfigProto_MatrixLayout_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull GemmConfigProto_MatrixLayout_class_data_;
}  // namespace xla
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::xla::BlasLtEpilogueProto_internal_data_>
    internal::EnumTraitsImpl::value<::xla::BlasLtEpilogueProto>;
template <>
internal::EnumTraitsT<::xla::GemmConfigProto_MatrixLayout_Order_internal_data_>
    internal::EnumTraitsImpl::value<::xla::GemmConfigProto_MatrixLayout_Order>;
}  // namespace protobuf
}  // namespace google

namespace xla {
enum GemmConfigProto_MatrixLayout_Order : int {
  GemmConfigProto_MatrixLayout_Order_ORDER_UNKNOWN = 0,
  GemmConfigProto_MatrixLayout_Order_ORDER_ROW_MAJOR = 1,
  GemmConfigProto_MatrixLayout_Order_ORDER_COLUMN_MAJOR = 2,
  GemmConfigProto_MatrixLayout_Order_GemmConfigProto_MatrixLayout_Order_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  GemmConfigProto_MatrixLayout_Order_GemmConfigProto_MatrixLayout_Order_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

PROTOBUF_EXPORT extern const uint32_t GemmConfigProto_MatrixLayout_Order_internal_data_[];
inline constexpr GemmConfigProto_MatrixLayout_Order GemmConfigProto_MatrixLayout_Order_Order_MIN =
    static_cast<GemmConfigProto_MatrixLayout_Order>(0);
inline constexpr GemmConfigProto_MatrixLayout_Order GemmConfigProto_MatrixLayout_Order_Order_MAX =
    static_cast<GemmConfigProto_MatrixLayout_Order>(2);
inline bool GemmConfigProto_MatrixLayout_Order_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int GemmConfigProto_MatrixLayout_Order_Order_ARRAYSIZE = 2 + 1;
PROTOBUF_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL GemmConfigProto_MatrixLayout_Order_descriptor();
template <typename T>
const ::std::string& GemmConfigProto_MatrixLayout_Order_Name(T value) {
  static_assert(::std::is_same<T, GemmConfigProto_MatrixLayout_Order>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Order_Name().");
  return GemmConfigProto_MatrixLayout_Order_Name(static_cast<GemmConfigProto_MatrixLayout_Order>(value));
}
template <>
inline const ::std::string& GemmConfigProto_MatrixLayout_Order_Name(GemmConfigProto_MatrixLayout_Order value) {
  return ::google::protobuf::internal::NameOfDenseEnum<GemmConfigProto_MatrixLayout_Order_descriptor, 0, 2>(
      static_cast<int>(value));
}
inline bool GemmConfigProto_MatrixLayout_Order_Parse(
    ::absl::string_view name, GemmConfigProto_MatrixLayout_Order* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<GemmConfigProto_MatrixLayout_Order>(GemmConfigProto_MatrixLayout_Order_descriptor(), name,
                                           value);
}
enum BlasLtEpilogueProto : int {
  EPILOGUE_DEFAULT = 0,
  EPILOGUE_RELU = 1,
  EPILOGUE_BIAS = 2,
  EPILOGUE_BIAS_THEN_RELU = 3,
  EPILOGUE_GELU = 4,
  EPILOGUE_SILU = 5,
  EPILOGUE_SILU_WITH_AUX = 6,
  EPILOGUE_GELU_WITH_AUX = 7,
  EPILOGUE_BIAS_THEN_GELU = 8,
  EPILOGUE_BIAS_THEN_SILU = 9,
  EPILOGUE_BIAS_THEN_GELU_WITH_AUX = 10,
  EPILOGUE_BIAS_THEN_SILU_WITH_AUX = 11,
  BlasLtEpilogueProto_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  BlasLtEpilogueProto_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

PROTOBUF_EXPORT extern const uint32_t BlasLtEpilogueProto_internal_data_[];
inline constexpr BlasLtEpilogueProto BlasLtEpilogueProto_MIN =
    static_cast<BlasLtEpilogueProto>(0);
inline constexpr BlasLtEpilogueProto BlasLtEpilogueProto_MAX =
    static_cast<BlasLtEpilogueProto>(11);
inline bool BlasLtEpilogueProto_IsValid(int value) {
  return 0 <= value && value <= 11;
}
inline constexpr int BlasLtEpilogueProto_ARRAYSIZE = 11 + 1;
PROTOBUF_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL BlasLtEpilogueProto_descriptor();
template <typename T>
const ::std::string& BlasLtEpilogueProto_Name(T value) {
  static_assert(::std::is_same<T, BlasLtEpilogueProto>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to BlasLtEpilogueProto_Name().");
  return BlasLtEpilogueProto_Name(static_cast<BlasLtEpilogueProto>(value));
}
template <>
inline const ::std::string& BlasLtEpilogueProto_Name(BlasLtEpilogueProto value) {
  return ::google::protobuf::internal::NameOfDenseEnum<BlasLtEpilogueProto_descriptor, 0, 11>(
      static_cast<int>(value));
}
inline bool BlasLtEpilogueProto_Parse(
    ::absl::string_view name, BlasLtEpilogueProto* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<BlasLtEpilogueProto>(BlasLtEpilogueProto_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT GemmConfigProto_MatrixLayout final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.GemmConfigProto.MatrixLayout) */ {
 public:
  inline GemmConfigProto_MatrixLayout() : GemmConfigProto_MatrixLayout(nullptr) {}
  ~GemmConfigProto_MatrixLayout() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(GemmConfigProto_MatrixLayout));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR GemmConfigProto_MatrixLayout(::google::protobuf::internal::ConstantInitialized);

  inline GemmConfigProto_MatrixLayout(const GemmConfigProto_MatrixLayout& from) : GemmConfigProto_MatrixLayout(nullptr, from) {}
  inline GemmConfigProto_MatrixLayout(GemmConfigProto_MatrixLayout&& from) noexcept
      : GemmConfigProto_MatrixLayout(nullptr, ::std::move(from)) {}
  inline GemmConfigProto_MatrixLayout& operator=(const GemmConfigProto_MatrixLayout& from) {
    CopyFrom(from);
    return *this;
  }
  inline GemmConfigProto_MatrixLayout& operator=(GemmConfigProto_MatrixLayout&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const GemmConfigProto_MatrixLayout& default_instance() {
    return *reinterpret_cast<const GemmConfigProto_MatrixLayout*>(
        &_GemmConfigProto_MatrixLayout_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(GemmConfigProto_MatrixLayout& a, GemmConfigProto_MatrixLayout& b) { a.Swap(&b); }
  inline void Swap(GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<GemmConfigProto_MatrixLayout>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const GemmConfigProto_MatrixLayout& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const GemmConfigProto_MatrixLayout& from) { GemmConfigProto_MatrixLayout::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "xla.GemmConfigProto.MatrixLayout"; }

 protected:
  explicit GemmConfigProto_MatrixLayout(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  GemmConfigProto_MatrixLayout(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const GemmConfigProto_MatrixLayout& from);
  GemmConfigProto_MatrixLayout(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, GemmConfigProto_MatrixLayout&& from) noexcept
      : GemmConfigProto_MatrixLayout(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Order = GemmConfigProto_MatrixLayout_Order;
  static constexpr Order ORDER_UNKNOWN = GemmConfigProto_MatrixLayout_Order_ORDER_UNKNOWN;
  static constexpr Order ORDER_ROW_MAJOR = GemmConfigProto_MatrixLayout_Order_ORDER_ROW_MAJOR;
  static constexpr Order ORDER_COLUMN_MAJOR = GemmConfigProto_MatrixLayout_Order_ORDER_COLUMN_MAJOR;
  static inline bool Order_IsValid(int value) {
    return GemmConfigProto_MatrixLayout_Order_IsValid(value);
  }
  static constexpr Order Order_MIN = GemmConfigProto_MatrixLayout_Order_Order_MIN;
  static constexpr Order Order_MAX = GemmConfigProto_MatrixLayout_Order_Order_MAX;
  static constexpr int Order_ARRAYSIZE = GemmConfigProto_MatrixLayout_Order_Order_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Order_descriptor() {
    return GemmConfigProto_MatrixLayout_Order_descriptor();
  }
  template <typename T>
  static inline const ::std::string& Order_Name(T value) {
    return GemmConfigProto_MatrixLayout_Order_Name(value);
  }
  static inline bool Order_Parse(
      ::absl::string_view name, Order* PROTOBUF_NONNULL value) {
    return GemmConfigProto_MatrixLayout_Order_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kNumRowsFieldNumber = 2,
    kNumColsFieldNumber = 3,
    kBatchSizeFieldNumber = 4,
    kOrderFieldNumber = 1,
    kTransposeFieldNumber = 7,
    kLeadingDimStrideFieldNumber = 5,
    kBatchStrideFieldNumber = 6,
    kDtypeFieldNumber = 8,
  };
  // int64 num_rows = 2;
  void clear_num_rows() ;
  ::int64_t num_rows() const;
  void set_num_rows(::int64_t value);

  private:
  ::int64_t _internal_num_rows() const;
  void _internal_set_num_rows(::int64_t value);

  public:
  // int64 num_cols = 3;
  void clear_num_cols() ;
  ::int64_t num_cols() const;
  void set_num_cols(::int64_t value);

  private:
  ::int64_t _internal_num_cols() const;
  void _internal_set_num_cols(::int64_t value);

  public:
  // int64 batch_size = 4;
  void clear_batch_size() ;
  ::int64_t batch_size() const;
  void set_batch_size(::int64_t value);

  private:
  ::int64_t _internal_batch_size() const;
  void _internal_set_batch_size(::int64_t value);

  public:
  // .xla.GemmConfigProto.MatrixLayout.Order order = 1;
  void clear_order() ;
  ::xla::GemmConfigProto_MatrixLayout_Order order() const;
  void set_order(::xla::GemmConfigProto_MatrixLayout_Order value);

  private:
  ::xla::GemmConfigProto_MatrixLayout_Order _internal_order() const;
  void _internal_set_order(::xla::GemmConfigProto_MatrixLayout_Order value);

  public:
  // .xla.BlasTransposeProto transpose = 7;
  void clear_transpose() ;
  ::xla::BlasTransposeProto transpose() const;
  void set_transpose(::xla::BlasTransposeProto value);

  private:
  ::xla::BlasTransposeProto _internal_transpose() const;
  void _internal_set_transpose(::xla::BlasTransposeProto value);

  public:
  // int64 leading_dim_stride = 5;
  void clear_leading_dim_stride() ;
  ::int64_t leading_dim_stride() const;
  void set_leading_dim_stride(::int64_t value);

  private:
  ::int64_t _internal_leading_dim_stride() const;
  void _internal_set_leading_dim_stride(::int64_t value);

  public:
  // int64 batch_stride = 6;
  void clear_batch_stride() ;
  ::int64_t batch_stride() const;
  void set_batch_stride(::int64_t value);

  private:
  ::int64_t _internal_batch_stride() const;
  void _internal_set_batch_stride(::int64_t value);

  public:
  // .xla.PrimitiveType dtype = 8;
  void clear_dtype() ;
  ::xla::PrimitiveType dtype() const;
  void set_dtype(::xla::PrimitiveType value);

  private:
  ::xla::PrimitiveType _internal_dtype() const;
  void _internal_set_dtype(::xla::PrimitiveType value);

  public:
  // @@protoc_insertion_point(class_scope:xla.GemmConfigProto.MatrixLayout)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 8,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const GemmConfigProto_MatrixLayout& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int64_t num_rows_;
    ::int64_t num_cols_;
    ::int64_t batch_size_;
    int order_;
    int transpose_;
    ::int64_t leading_dim_stride_;
    ::int64_t batch_stride_;
    int dtype_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fstream_5fexecutor_2fgpu_2fgpu_5fblas_5flt_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull GemmConfigProto_MatrixLayout_class_data_;
// -------------------------------------------------------------------

class PROTOBUF_EXPORT GemmConfigProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:xla.GemmConfigProto) */ {
 public:
  inline GemmConfigProto() : GemmConfigProto(nullptr) {}
  ~GemmConfigProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(GemmConfigProto* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(GemmConfigProto));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR GemmConfigProto(::google::protobuf::internal::ConstantInitialized);

  inline GemmConfigProto(const GemmConfigProto& from) : GemmConfigProto(nullptr, from) {}
  inline GemmConfigProto(GemmConfigProto&& from) noexcept
      : GemmConfigProto(nullptr, ::std::move(from)) {}
  inline GemmConfigProto& operator=(const GemmConfigProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline GemmConfigProto& operator=(GemmConfigProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const GemmConfigProto& default_instance() {
    return *reinterpret_cast<const GemmConfigProto*>(
        &_GemmConfigProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(GemmConfigProto& a, GemmConfigProto& b) { a.Swap(&b); }
  inline void Swap(GemmConfigProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(GemmConfigProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  GemmConfigProto* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<GemmConfigProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const GemmConfigProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const GemmConfigProto& from) { GemmConfigProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(GemmConfigProto* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "xla.GemmConfigProto"; }

 protected:
  explicit GemmConfigProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  GemmConfigProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const GemmConfigProto& from);
  GemmConfigProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, GemmConfigProto&& from) noexcept
      : GemmConfigProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using MatrixLayout = GemmConfigProto_MatrixLayout;

  // accessors -------------------------------------------------------
  enum : int {
    kLhsLayoutFieldNumber = 1,
    kRhsLayoutFieldNumber = 2,
    kCLayoutFieldNumber = 3,
    kOutputLayoutFieldNumber = 4,
    kAlphaRealFieldNumber = 5,
    kAlphaImagFieldNumber = 6,
    kBetaFieldNumber = 7,
    kComputePrecisionFieldNumber = 8,
    kAlgorithmFieldNumber = 10,
    kPrecisionAlgorithmFieldNumber = 9,
    kGradXFieldNumber = 11,
    kGradYFieldNumber = 12,
    kComputeTypeFieldNumber = 13,
  };
  // .xla.GemmConfigProto.MatrixLayout lhs_layout = 1;
  bool has_lhs_layout() const;
  void clear_lhs_layout() ;
  const ::xla::GemmConfigProto_MatrixLayout& lhs_layout() const;
  [[nodiscard]] ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE release_lhs_layout();
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL mutable_lhs_layout();
  void set_allocated_lhs_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_lhs_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value);
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE unsafe_arena_release_lhs_layout();

  private:
  const ::xla::GemmConfigProto_MatrixLayout& _internal_lhs_layout() const;
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL _internal_mutable_lhs_layout();

  public:
  // .xla.GemmConfigProto.MatrixLayout rhs_layout = 2;
  bool has_rhs_layout() const;
  void clear_rhs_layout() ;
  const ::xla::GemmConfigProto_MatrixLayout& rhs_layout() const;
  [[nodiscard]] ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE release_rhs_layout();
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL mutable_rhs_layout();
  void set_allocated_rhs_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_rhs_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value);
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE unsafe_arena_release_rhs_layout();

  private:
  const ::xla::GemmConfigProto_MatrixLayout& _internal_rhs_layout() const;
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL _internal_mutable_rhs_layout();

  public:
  // .xla.GemmConfigProto.MatrixLayout c_layout = 3;
  bool has_c_layout() const;
  void clear_c_layout() ;
  const ::xla::GemmConfigProto_MatrixLayout& c_layout() const;
  [[nodiscard]] ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE release_c_layout();
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL mutable_c_layout();
  void set_allocated_c_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_c_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value);
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE unsafe_arena_release_c_layout();

  private:
  const ::xla::GemmConfigProto_MatrixLayout& _internal_c_layout() const;
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL _internal_mutable_c_layout();

  public:
  // .xla.GemmConfigProto.MatrixLayout output_layout = 4;
  bool has_output_layout() const;
  void clear_output_layout() ;
  const ::xla::GemmConfigProto_MatrixLayout& output_layout() const;
  [[nodiscard]] ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE release_output_layout();
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL mutable_output_layout();
  void set_allocated_output_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_output_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value);
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE unsafe_arena_release_output_layout();

  private:
  const ::xla::GemmConfigProto_MatrixLayout& _internal_output_layout() const;
  ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL _internal_mutable_output_layout();

  public:
  // double alpha_real = 5;
  void clear_alpha_real() ;
  double alpha_real() const;
  void set_alpha_real(double value);

  private:
  double _internal_alpha_real() const;
  void _internal_set_alpha_real(double value);

  public:
  // double alpha_imag = 6;
  void clear_alpha_imag() ;
  double alpha_imag() const;
  void set_alpha_imag(double value);

  private:
  double _internal_alpha_imag() const;
  void _internal_set_alpha_imag(double value);

  public:
  // double beta = 7;
  void clear_beta() ;
  double beta() const;
  void set_beta(double value);

  private:
  double _internal_beta() const;
  void _internal_set_beta(double value);

  public:
  // int64 compute_precision = 8;
  void clear_compute_precision() ;
  ::int64_t compute_precision() const;
  void set_compute_precision(::int64_t value);

  private:
  ::int64_t _internal_compute_precision() const;
  void _internal_set_compute_precision(::int64_t value);

  public:
  // optional int64 algorithm = 10;
  bool has_algorithm() const;
  void clear_algorithm() ;
  ::int64_t algorithm() const;
  void set_algorithm(::int64_t value);

  private:
  ::int64_t _internal_algorithm() const;
  void _internal_set_algorithm(::int64_t value);

  public:
  // .xla.PrecisionConfig.Algorithm precision_algorithm = 9;
  void clear_precision_algorithm() ;
  ::xla::PrecisionConfig_Algorithm precision_algorithm() const;
  void set_precision_algorithm(::xla::PrecisionConfig_Algorithm value);

  private:
  ::xla::PrecisionConfig_Algorithm _internal_precision_algorithm() const;
  void _internal_set_precision_algorithm(::xla::PrecisionConfig_Algorithm value);

  public:
  // bool grad_x = 11;
  void clear_grad_x() ;
  bool grad_x() const;
  void set_grad_x(bool value);

  private:
  bool _internal_grad_x() const;
  void _internal_set_grad_x(bool value);

  public:
  // bool grad_y = 12;
  void clear_grad_y() ;
  bool grad_y() const;
  void set_grad_y(bool value);

  private:
  bool _internal_grad_y() const;
  void _internal_set_grad_y(bool value);

  public:
  // .xla.BlasComputationTypeProto compute_type = 13;
  void clear_compute_type() ;
  ::xla::BlasComputationTypeProto compute_type() const;
  void set_compute_type(::xla::BlasComputationTypeProto value);

  private:
  ::xla::BlasComputationTypeProto _internal_compute_type() const;
  void _internal_set_compute_type(::xla::BlasComputationTypeProto value);

  public:
  // @@protoc_insertion_point(class_scope:xla.GemmConfigProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 13,
                                   4, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const GemmConfigProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE lhs_layout_;
    ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE rhs_layout_;
    ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE c_layout_;
    ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE output_layout_;
    double alpha_real_;
    double alpha_imag_;
    double beta_;
    ::int64_t compute_precision_;
    ::int64_t algorithm_;
    int precision_algorithm_;
    bool grad_x_;
    bool grad_y_;
    int compute_type_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fstream_5fexecutor_2fgpu_2fgpu_5fblas_5flt_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull GemmConfigProto_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// GemmConfigProto_MatrixLayout

// .xla.GemmConfigProto.MatrixLayout.Order order = 1;
inline void GemmConfigProto_MatrixLayout::clear_order() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.order_ = 0;
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline ::xla::GemmConfigProto_MatrixLayout_Order GemmConfigProto_MatrixLayout::order() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.MatrixLayout.order)
  return _internal_order();
}
inline void GemmConfigProto_MatrixLayout::set_order(::xla::GemmConfigProto_MatrixLayout_Order value) {
  _internal_set_order(value);
  _impl_._has_bits_[0] |= 0x00000008u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.MatrixLayout.order)
}
inline ::xla::GemmConfigProto_MatrixLayout_Order GemmConfigProto_MatrixLayout::_internal_order() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::xla::GemmConfigProto_MatrixLayout_Order>(_impl_.order_);
}
inline void GemmConfigProto_MatrixLayout::_internal_set_order(::xla::GemmConfigProto_MatrixLayout_Order value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.order_ = value;
}

// int64 num_rows = 2;
inline void GemmConfigProto_MatrixLayout::clear_num_rows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.num_rows_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline ::int64_t GemmConfigProto_MatrixLayout::num_rows() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.MatrixLayout.num_rows)
  return _internal_num_rows();
}
inline void GemmConfigProto_MatrixLayout::set_num_rows(::int64_t value) {
  _internal_set_num_rows(value);
  _impl_._has_bits_[0] |= 0x00000001u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.MatrixLayout.num_rows)
}
inline ::int64_t GemmConfigProto_MatrixLayout::_internal_num_rows() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.num_rows_;
}
inline void GemmConfigProto_MatrixLayout::_internal_set_num_rows(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.num_rows_ = value;
}

// int64 num_cols = 3;
inline void GemmConfigProto_MatrixLayout::clear_num_cols() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.num_cols_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline ::int64_t GemmConfigProto_MatrixLayout::num_cols() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.MatrixLayout.num_cols)
  return _internal_num_cols();
}
inline void GemmConfigProto_MatrixLayout::set_num_cols(::int64_t value) {
  _internal_set_num_cols(value);
  _impl_._has_bits_[0] |= 0x00000002u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.MatrixLayout.num_cols)
}
inline ::int64_t GemmConfigProto_MatrixLayout::_internal_num_cols() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.num_cols_;
}
inline void GemmConfigProto_MatrixLayout::_internal_set_num_cols(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.num_cols_ = value;
}

// int64 batch_size = 4;
inline void GemmConfigProto_MatrixLayout::clear_batch_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.batch_size_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline ::int64_t GemmConfigProto_MatrixLayout::batch_size() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.MatrixLayout.batch_size)
  return _internal_batch_size();
}
inline void GemmConfigProto_MatrixLayout::set_batch_size(::int64_t value) {
  _internal_set_batch_size(value);
  _impl_._has_bits_[0] |= 0x00000004u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.MatrixLayout.batch_size)
}
inline ::int64_t GemmConfigProto_MatrixLayout::_internal_batch_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.batch_size_;
}
inline void GemmConfigProto_MatrixLayout::_internal_set_batch_size(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.batch_size_ = value;
}

// int64 leading_dim_stride = 5;
inline void GemmConfigProto_MatrixLayout::clear_leading_dim_stride() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.leading_dim_stride_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000020u;
}
inline ::int64_t GemmConfigProto_MatrixLayout::leading_dim_stride() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.MatrixLayout.leading_dim_stride)
  return _internal_leading_dim_stride();
}
inline void GemmConfigProto_MatrixLayout::set_leading_dim_stride(::int64_t value) {
  _internal_set_leading_dim_stride(value);
  _impl_._has_bits_[0] |= 0x00000020u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.MatrixLayout.leading_dim_stride)
}
inline ::int64_t GemmConfigProto_MatrixLayout::_internal_leading_dim_stride() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.leading_dim_stride_;
}
inline void GemmConfigProto_MatrixLayout::_internal_set_leading_dim_stride(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.leading_dim_stride_ = value;
}

// int64 batch_stride = 6;
inline void GemmConfigProto_MatrixLayout::clear_batch_stride() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.batch_stride_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000040u;
}
inline ::int64_t GemmConfigProto_MatrixLayout::batch_stride() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.MatrixLayout.batch_stride)
  return _internal_batch_stride();
}
inline void GemmConfigProto_MatrixLayout::set_batch_stride(::int64_t value) {
  _internal_set_batch_stride(value);
  _impl_._has_bits_[0] |= 0x00000040u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.MatrixLayout.batch_stride)
}
inline ::int64_t GemmConfigProto_MatrixLayout::_internal_batch_stride() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.batch_stride_;
}
inline void GemmConfigProto_MatrixLayout::_internal_set_batch_stride(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.batch_stride_ = value;
}

// .xla.BlasTransposeProto transpose = 7;
inline void GemmConfigProto_MatrixLayout::clear_transpose() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.transpose_ = 0;
  _impl_._has_bits_[0] &= ~0x00000010u;
}
inline ::xla::BlasTransposeProto GemmConfigProto_MatrixLayout::transpose() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.MatrixLayout.transpose)
  return _internal_transpose();
}
inline void GemmConfigProto_MatrixLayout::set_transpose(::xla::BlasTransposeProto value) {
  _internal_set_transpose(value);
  _impl_._has_bits_[0] |= 0x00000010u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.MatrixLayout.transpose)
}
inline ::xla::BlasTransposeProto GemmConfigProto_MatrixLayout::_internal_transpose() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::xla::BlasTransposeProto>(_impl_.transpose_);
}
inline void GemmConfigProto_MatrixLayout::_internal_set_transpose(::xla::BlasTransposeProto value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.transpose_ = value;
}

// .xla.PrimitiveType dtype = 8;
inline void GemmConfigProto_MatrixLayout::clear_dtype() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dtype_ = 0;
  _impl_._has_bits_[0] &= ~0x00000080u;
}
inline ::xla::PrimitiveType GemmConfigProto_MatrixLayout::dtype() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.MatrixLayout.dtype)
  return _internal_dtype();
}
inline void GemmConfigProto_MatrixLayout::set_dtype(::xla::PrimitiveType value) {
  _internal_set_dtype(value);
  _impl_._has_bits_[0] |= 0x00000080u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.MatrixLayout.dtype)
}
inline ::xla::PrimitiveType GemmConfigProto_MatrixLayout::_internal_dtype() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::xla::PrimitiveType>(_impl_.dtype_);
}
inline void GemmConfigProto_MatrixLayout::_internal_set_dtype(::xla::PrimitiveType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.dtype_ = value;
}

// -------------------------------------------------------------------

// GemmConfigProto

// .xla.GemmConfigProto.MatrixLayout lhs_layout = 1;
inline bool GemmConfigProto::has_lhs_layout() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.lhs_layout_ != nullptr);
  return value;
}
inline void GemmConfigProto::clear_lhs_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.lhs_layout_ != nullptr) _impl_.lhs_layout_->Clear();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::xla::GemmConfigProto_MatrixLayout& GemmConfigProto::_internal_lhs_layout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::xla::GemmConfigProto_MatrixLayout* p = _impl_.lhs_layout_;
  return p != nullptr ? *p : reinterpret_cast<const ::xla::GemmConfigProto_MatrixLayout&>(::xla::_GemmConfigProto_MatrixLayout_default_instance_);
}
inline const ::xla::GemmConfigProto_MatrixLayout& GemmConfigProto::lhs_layout() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.lhs_layout)
  return _internal_lhs_layout();
}
inline void GemmConfigProto::unsafe_arena_set_allocated_lhs_layout(
    ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.lhs_layout_);
  }
  _impl_.lhs_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:xla.GemmConfigProto.lhs_layout)
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE GemmConfigProto::release_lhs_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000001u;
  ::xla::GemmConfigProto_MatrixLayout* released = _impl_.lhs_layout_;
  _impl_.lhs_layout_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE GemmConfigProto::unsafe_arena_release_lhs_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:xla.GemmConfigProto.lhs_layout)

  _impl_._has_bits_[0] &= ~0x00000001u;
  ::xla::GemmConfigProto_MatrixLayout* temp = _impl_.lhs_layout_;
  _impl_.lhs_layout_ = nullptr;
  return temp;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL GemmConfigProto::_internal_mutable_lhs_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.lhs_layout_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::xla::GemmConfigProto_MatrixLayout>(GetArena());
    _impl_.lhs_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(p);
  }
  return _impl_.lhs_layout_;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL GemmConfigProto::mutable_lhs_layout()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000001u;
  ::xla::GemmConfigProto_MatrixLayout* _msg = _internal_mutable_lhs_layout();
  // @@protoc_insertion_point(field_mutable:xla.GemmConfigProto.lhs_layout)
  return _msg;
}
inline void GemmConfigProto::set_allocated_lhs_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.lhs_layout_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }

  _impl_.lhs_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(value);
  // @@protoc_insertion_point(field_set_allocated:xla.GemmConfigProto.lhs_layout)
}

// .xla.GemmConfigProto.MatrixLayout rhs_layout = 2;
inline bool GemmConfigProto::has_rhs_layout() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.rhs_layout_ != nullptr);
  return value;
}
inline void GemmConfigProto::clear_rhs_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.rhs_layout_ != nullptr) _impl_.rhs_layout_->Clear();
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline const ::xla::GemmConfigProto_MatrixLayout& GemmConfigProto::_internal_rhs_layout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::xla::GemmConfigProto_MatrixLayout* p = _impl_.rhs_layout_;
  return p != nullptr ? *p : reinterpret_cast<const ::xla::GemmConfigProto_MatrixLayout&>(::xla::_GemmConfigProto_MatrixLayout_default_instance_);
}
inline const ::xla::GemmConfigProto_MatrixLayout& GemmConfigProto::rhs_layout() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.rhs_layout)
  return _internal_rhs_layout();
}
inline void GemmConfigProto::unsafe_arena_set_allocated_rhs_layout(
    ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.rhs_layout_);
  }
  _impl_.rhs_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:xla.GemmConfigProto.rhs_layout)
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE GemmConfigProto::release_rhs_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000002u;
  ::xla::GemmConfigProto_MatrixLayout* released = _impl_.rhs_layout_;
  _impl_.rhs_layout_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE GemmConfigProto::unsafe_arena_release_rhs_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:xla.GemmConfigProto.rhs_layout)

  _impl_._has_bits_[0] &= ~0x00000002u;
  ::xla::GemmConfigProto_MatrixLayout* temp = _impl_.rhs_layout_;
  _impl_.rhs_layout_ = nullptr;
  return temp;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL GemmConfigProto::_internal_mutable_rhs_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.rhs_layout_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::xla::GemmConfigProto_MatrixLayout>(GetArena());
    _impl_.rhs_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(p);
  }
  return _impl_.rhs_layout_;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL GemmConfigProto::mutable_rhs_layout()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000002u;
  ::xla::GemmConfigProto_MatrixLayout* _msg = _internal_mutable_rhs_layout();
  // @@protoc_insertion_point(field_mutable:xla.GemmConfigProto.rhs_layout)
  return _msg;
}
inline void GemmConfigProto::set_allocated_rhs_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.rhs_layout_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }

  _impl_.rhs_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(value);
  // @@protoc_insertion_point(field_set_allocated:xla.GemmConfigProto.rhs_layout)
}

// .xla.GemmConfigProto.MatrixLayout c_layout = 3;
inline bool GemmConfigProto::has_c_layout() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.c_layout_ != nullptr);
  return value;
}
inline void GemmConfigProto::clear_c_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.c_layout_ != nullptr) _impl_.c_layout_->Clear();
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline const ::xla::GemmConfigProto_MatrixLayout& GemmConfigProto::_internal_c_layout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::xla::GemmConfigProto_MatrixLayout* p = _impl_.c_layout_;
  return p != nullptr ? *p : reinterpret_cast<const ::xla::GemmConfigProto_MatrixLayout&>(::xla::_GemmConfigProto_MatrixLayout_default_instance_);
}
inline const ::xla::GemmConfigProto_MatrixLayout& GemmConfigProto::c_layout() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.c_layout)
  return _internal_c_layout();
}
inline void GemmConfigProto::unsafe_arena_set_allocated_c_layout(
    ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.c_layout_);
  }
  _impl_.c_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:xla.GemmConfigProto.c_layout)
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE GemmConfigProto::release_c_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000004u;
  ::xla::GemmConfigProto_MatrixLayout* released = _impl_.c_layout_;
  _impl_.c_layout_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE GemmConfigProto::unsafe_arena_release_c_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:xla.GemmConfigProto.c_layout)

  _impl_._has_bits_[0] &= ~0x00000004u;
  ::xla::GemmConfigProto_MatrixLayout* temp = _impl_.c_layout_;
  _impl_.c_layout_ = nullptr;
  return temp;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL GemmConfigProto::_internal_mutable_c_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.c_layout_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::xla::GemmConfigProto_MatrixLayout>(GetArena());
    _impl_.c_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(p);
  }
  return _impl_.c_layout_;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL GemmConfigProto::mutable_c_layout()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000004u;
  ::xla::GemmConfigProto_MatrixLayout* _msg = _internal_mutable_c_layout();
  // @@protoc_insertion_point(field_mutable:xla.GemmConfigProto.c_layout)
  return _msg;
}
inline void GemmConfigProto::set_allocated_c_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.c_layout_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }

  _impl_.c_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(value);
  // @@protoc_insertion_point(field_set_allocated:xla.GemmConfigProto.c_layout)
}

// .xla.GemmConfigProto.MatrixLayout output_layout = 4;
inline bool GemmConfigProto::has_output_layout() const {
  bool value = (_impl_._has_bits_[0] & 0x00000008u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.output_layout_ != nullptr);
  return value;
}
inline void GemmConfigProto::clear_output_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.output_layout_ != nullptr) _impl_.output_layout_->Clear();
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline const ::xla::GemmConfigProto_MatrixLayout& GemmConfigProto::_internal_output_layout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::xla::GemmConfigProto_MatrixLayout* p = _impl_.output_layout_;
  return p != nullptr ? *p : reinterpret_cast<const ::xla::GemmConfigProto_MatrixLayout&>(::xla::_GemmConfigProto_MatrixLayout_default_instance_);
}
inline const ::xla::GemmConfigProto_MatrixLayout& GemmConfigProto::output_layout() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.output_layout)
  return _internal_output_layout();
}
inline void GemmConfigProto::unsafe_arena_set_allocated_output_layout(
    ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.output_layout_);
  }
  _impl_.output_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000008u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000008u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:xla.GemmConfigProto.output_layout)
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE GemmConfigProto::release_output_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000008u;
  ::xla::GemmConfigProto_MatrixLayout* released = _impl_.output_layout_;
  _impl_.output_layout_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE GemmConfigProto::unsafe_arena_release_output_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:xla.GemmConfigProto.output_layout)

  _impl_._has_bits_[0] &= ~0x00000008u;
  ::xla::GemmConfigProto_MatrixLayout* temp = _impl_.output_layout_;
  _impl_.output_layout_ = nullptr;
  return temp;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL GemmConfigProto::_internal_mutable_output_layout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.output_layout_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::xla::GemmConfigProto_MatrixLayout>(GetArena());
    _impl_.output_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(p);
  }
  return _impl_.output_layout_;
}
inline ::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NONNULL GemmConfigProto::mutable_output_layout()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000008u;
  ::xla::GemmConfigProto_MatrixLayout* _msg = _internal_mutable_output_layout();
  // @@protoc_insertion_point(field_mutable:xla.GemmConfigProto.output_layout)
  return _msg;
}
inline void GemmConfigProto::set_allocated_output_layout(::xla::GemmConfigProto_MatrixLayout* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.output_layout_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000008u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000008u;
  }

  _impl_.output_layout_ = reinterpret_cast<::xla::GemmConfigProto_MatrixLayout*>(value);
  // @@protoc_insertion_point(field_set_allocated:xla.GemmConfigProto.output_layout)
}

// double alpha_real = 5;
inline void GemmConfigProto::clear_alpha_real() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alpha_real_ = 0;
  _impl_._has_bits_[0] &= ~0x00000010u;
}
inline double GemmConfigProto::alpha_real() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.alpha_real)
  return _internal_alpha_real();
}
inline void GemmConfigProto::set_alpha_real(double value) {
  _internal_set_alpha_real(value);
  _impl_._has_bits_[0] |= 0x00000010u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.alpha_real)
}
inline double GemmConfigProto::_internal_alpha_real() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.alpha_real_;
}
inline void GemmConfigProto::_internal_set_alpha_real(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alpha_real_ = value;
}

// double alpha_imag = 6;
inline void GemmConfigProto::clear_alpha_imag() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alpha_imag_ = 0;
  _impl_._has_bits_[0] &= ~0x00000020u;
}
inline double GemmConfigProto::alpha_imag() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.alpha_imag)
  return _internal_alpha_imag();
}
inline void GemmConfigProto::set_alpha_imag(double value) {
  _internal_set_alpha_imag(value);
  _impl_._has_bits_[0] |= 0x00000020u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.alpha_imag)
}
inline double GemmConfigProto::_internal_alpha_imag() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.alpha_imag_;
}
inline void GemmConfigProto::_internal_set_alpha_imag(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alpha_imag_ = value;
}

// double beta = 7;
inline void GemmConfigProto::clear_beta() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.beta_ = 0;
  _impl_._has_bits_[0] &= ~0x00000040u;
}
inline double GemmConfigProto::beta() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.beta)
  return _internal_beta();
}
inline void GemmConfigProto::set_beta(double value) {
  _internal_set_beta(value);
  _impl_._has_bits_[0] |= 0x00000040u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.beta)
}
inline double GemmConfigProto::_internal_beta() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.beta_;
}
inline void GemmConfigProto::_internal_set_beta(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.beta_ = value;
}

// int64 compute_precision = 8;
inline void GemmConfigProto::clear_compute_precision() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.compute_precision_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000080u;
}
inline ::int64_t GemmConfigProto::compute_precision() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.compute_precision)
  return _internal_compute_precision();
}
inline void GemmConfigProto::set_compute_precision(::int64_t value) {
  _internal_set_compute_precision(value);
  _impl_._has_bits_[0] |= 0x00000080u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.compute_precision)
}
inline ::int64_t GemmConfigProto::_internal_compute_precision() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.compute_precision_;
}
inline void GemmConfigProto::_internal_set_compute_precision(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.compute_precision_ = value;
}

// .xla.PrecisionConfig.Algorithm precision_algorithm = 9;
inline void GemmConfigProto::clear_precision_algorithm() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.precision_algorithm_ = 0;
  _impl_._has_bits_[0] &= ~0x00000200u;
}
inline ::xla::PrecisionConfig_Algorithm GemmConfigProto::precision_algorithm() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.precision_algorithm)
  return _internal_precision_algorithm();
}
inline void GemmConfigProto::set_precision_algorithm(::xla::PrecisionConfig_Algorithm value) {
  _internal_set_precision_algorithm(value);
  _impl_._has_bits_[0] |= 0x00000200u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.precision_algorithm)
}
inline ::xla::PrecisionConfig_Algorithm GemmConfigProto::_internal_precision_algorithm() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::xla::PrecisionConfig_Algorithm>(_impl_.precision_algorithm_);
}
inline void GemmConfigProto::_internal_set_precision_algorithm(::xla::PrecisionConfig_Algorithm value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.precision_algorithm_ = value;
}

// optional int64 algorithm = 10;
inline bool GemmConfigProto::has_algorithm() const {
  bool value = (_impl_._has_bits_[0] & 0x00000100u) != 0;
  return value;
}
inline void GemmConfigProto::clear_algorithm() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.algorithm_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000100u;
}
inline ::int64_t GemmConfigProto::algorithm() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.algorithm)
  return _internal_algorithm();
}
inline void GemmConfigProto::set_algorithm(::int64_t value) {
  _internal_set_algorithm(value);
  _impl_._has_bits_[0] |= 0x00000100u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.algorithm)
}
inline ::int64_t GemmConfigProto::_internal_algorithm() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.algorithm_;
}
inline void GemmConfigProto::_internal_set_algorithm(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.algorithm_ = value;
}

// bool grad_x = 11;
inline void GemmConfigProto::clear_grad_x() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.grad_x_ = false;
  _impl_._has_bits_[0] &= ~0x00000400u;
}
inline bool GemmConfigProto::grad_x() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.grad_x)
  return _internal_grad_x();
}
inline void GemmConfigProto::set_grad_x(bool value) {
  _internal_set_grad_x(value);
  _impl_._has_bits_[0] |= 0x00000400u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.grad_x)
}
inline bool GemmConfigProto::_internal_grad_x() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.grad_x_;
}
inline void GemmConfigProto::_internal_set_grad_x(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.grad_x_ = value;
}

// bool grad_y = 12;
inline void GemmConfigProto::clear_grad_y() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.grad_y_ = false;
  _impl_._has_bits_[0] &= ~0x00000800u;
}
inline bool GemmConfigProto::grad_y() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.grad_y)
  return _internal_grad_y();
}
inline void GemmConfigProto::set_grad_y(bool value) {
  _internal_set_grad_y(value);
  _impl_._has_bits_[0] |= 0x00000800u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.grad_y)
}
inline bool GemmConfigProto::_internal_grad_y() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.grad_y_;
}
inline void GemmConfigProto::_internal_set_grad_y(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.grad_y_ = value;
}

// .xla.BlasComputationTypeProto compute_type = 13;
inline void GemmConfigProto::clear_compute_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.compute_type_ = 0;
  _impl_._has_bits_[0] &= ~0x00001000u;
}
inline ::xla::BlasComputationTypeProto GemmConfigProto::compute_type() const {
  // @@protoc_insertion_point(field_get:xla.GemmConfigProto.compute_type)
  return _internal_compute_type();
}
inline void GemmConfigProto::set_compute_type(::xla::BlasComputationTypeProto value) {
  _internal_set_compute_type(value);
  _impl_._has_bits_[0] |= 0x00001000u;
  // @@protoc_insertion_point(field_set:xla.GemmConfigProto.compute_type)
}
inline ::xla::BlasComputationTypeProto GemmConfigProto::_internal_compute_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::xla::BlasComputationTypeProto>(_impl_.compute_type_);
}
inline void GemmConfigProto::_internal_set_compute_type(::xla::BlasComputationTypeProto value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.compute_type_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace xla


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::xla::GemmConfigProto_MatrixLayout_Order> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::xla::GemmConfigProto_MatrixLayout_Order>() {
  return ::xla::GemmConfigProto_MatrixLayout_Order_descriptor();
}
template <>
struct is_proto_enum<::xla::BlasLtEpilogueProto> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::xla::BlasLtEpilogueProto>() {
  return ::xla::BlasLtEpilogueProto_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // xla_2fstream_5fexecutor_2fgpu_2fgpu_5fblas_5flt_2eproto_2epb_2eh
