// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: tensorflow/compiler/mlir/quantization/tensorflow/exported_model.proto
// Protobuf C++ Version: 6.31.1

#ifndef tensorflow_2fcompiler_2fmlir_2fquantization_2ftensorflow_2fexported_5fmodel_2eproto_2epb_2eh
#define tensorflow_2fcompiler_2fmlir_2fquantization_2ftensorflow_2fexported_5fmodel_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6031001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/map.h"  // IWYU pragma: export
#include "google/protobuf/map_type_handler.h"  // IWYU pragma: export
#include "google/protobuf/map_entry.h"
#include "google/protobuf/map_field.h"
#include "google/protobuf/unknown_field_set.h"
#include "tensorflow/core/framework/graph.pb.h"
#include "tensorflow/core/protobuf/meta_graph.pb.h"
#include "tensorflow/core/protobuf/saver.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_tensorflow_2fcompiler_2fmlir_2fquantization_2ftensorflow_2fexported_5fmodel_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_tensorflow_2fcompiler_2fmlir_2fquantization_2ftensorflow_2fexported_5fmodel_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable descriptor_table_tensorflow_2fcompiler_2fmlir_2fquantization_2ftensorflow_2fexported_5fmodel_2eproto;
}  // extern "C"
namespace tensorflow {
namespace quantization {
class ExportedModel;
struct ExportedModelDefaultTypeInternal;
PROTOBUF_EXPORT extern ExportedModelDefaultTypeInternal _ExportedModel_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull ExportedModel_class_data_;
class ExportedModel_FunctionAliasesEntry_DoNotUse;
struct ExportedModel_FunctionAliasesEntry_DoNotUseDefaultTypeInternal;
PROTOBUF_EXPORT extern ExportedModel_FunctionAliasesEntry_DoNotUseDefaultTypeInternal _ExportedModel_FunctionAliasesEntry_DoNotUse_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull ExportedModel_FunctionAliasesEntry_DoNotUse_class_data_;
}  // namespace quantization
}  // namespace tensorflow
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace tensorflow {
namespace quantization {

// ===================================================================


// -------------------------------------------------------------------

class ExportedModel_FunctionAliasesEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<std::string, std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<std::string, std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ExportedModel_FunctionAliasesEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExportedModel_FunctionAliasesEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ExportedModel_FunctionAliasesEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ExportedModel_FunctionAliasesEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_tensorflow_2fcompiler_2fmlir_2fquantization_2ftensorflow_2fexported_5fmodel_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 75,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull ExportedModel_FunctionAliasesEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PROTOBUF_EXPORT ExportedModel final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:tensorflow.quantization.ExportedModel) */ {
 public:
  inline ExportedModel() : ExportedModel(nullptr) {}
  ~ExportedModel() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExportedModel* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExportedModel));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExportedModel(::google::protobuf::internal::ConstantInitialized);

  inline ExportedModel(const ExportedModel& from) : ExportedModel(nullptr, from) {}
  inline ExportedModel(ExportedModel&& from) noexcept
      : ExportedModel(nullptr, ::std::move(from)) {}
  inline ExportedModel& operator=(const ExportedModel& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExportedModel& operator=(ExportedModel&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ExportedModel& default_instance() {
    return *reinterpret_cast<const ExportedModel*>(
        &_ExportedModel_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ExportedModel& a, ExportedModel& b) { a.Swap(&b); }
  inline void Swap(ExportedModel* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExportedModel* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ExportedModel* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExportedModel>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExportedModel& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExportedModel& from) { ExportedModel::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExportedModel* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "tensorflow.quantization.ExportedModel"; }

 protected:
  explicit ExportedModel(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExportedModel(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExportedModel& from);
  ExportedModel(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExportedModel&& from) noexcept
      : ExportedModel(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFunctionAliasesFieldNumber = 6,
    kAssetFileDefsFieldNumber = 8,
    kInitNodeNameFieldNumber = 2,
    kCheckpointDirFieldNumber = 5,
    kGraphDefFieldNumber = 1,
    kSaverDefFieldNumber = 10,
  };
  // map<string, string> function_aliases = 6;
  int function_aliases_size() const;
  private:
  int _internal_function_aliases_size() const;

  public:
  void clear_function_aliases() ;
  const ::google::protobuf::Map<std::string, std::string>& function_aliases() const;
  ::google::protobuf::Map<std::string, std::string>* PROTOBUF_NONNULL mutable_function_aliases();

  private:
  const ::google::protobuf::Map<std::string, std::string>& _internal_function_aliases() const;
  ::google::protobuf::Map<std::string, std::string>* PROTOBUF_NONNULL _internal_mutable_function_aliases();

  public:
  // repeated .tensorflow.AssetFileDef asset_file_defs = 8;
  int asset_file_defs_size() const;
  private:
  int _internal_asset_file_defs_size() const;

  public:
  void clear_asset_file_defs() ;
  ::tensorflow::AssetFileDef* PROTOBUF_NONNULL mutable_asset_file_defs(int index);
  ::google::protobuf::RepeatedPtrField<::tensorflow::AssetFileDef>* PROTOBUF_NONNULL mutable_asset_file_defs();

  private:
  const ::google::protobuf::RepeatedPtrField<::tensorflow::AssetFileDef>& _internal_asset_file_defs() const;
  ::google::protobuf::RepeatedPtrField<::tensorflow::AssetFileDef>* PROTOBUF_NONNULL _internal_mutable_asset_file_defs();
  public:
  const ::tensorflow::AssetFileDef& asset_file_defs(int index) const;
  ::tensorflow::AssetFileDef* PROTOBUF_NONNULL add_asset_file_defs();
  const ::google::protobuf::RepeatedPtrField<::tensorflow::AssetFileDef>& asset_file_defs() const;
  // string init_node_name = 2;
  void clear_init_node_name() ;
  const ::std::string& init_node_name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_init_node_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_init_node_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_init_node_name();
  void set_allocated_init_node_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_init_node_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_init_node_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_init_node_name();

  public:
  // string checkpoint_dir = 5;
  void clear_checkpoint_dir() ;
  const ::std::string& checkpoint_dir() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_checkpoint_dir(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_checkpoint_dir();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_checkpoint_dir();
  void set_allocated_checkpoint_dir(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_checkpoint_dir() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_checkpoint_dir(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_checkpoint_dir();

  public:
  // .tensorflow.GraphDef graph_def = 1;
  bool has_graph_def() const;
  void clear_graph_def() ;
  const ::tensorflow::GraphDef& graph_def() const;
  [[nodiscard]] ::tensorflow::GraphDef* PROTOBUF_NULLABLE release_graph_def();
  ::tensorflow::GraphDef* PROTOBUF_NONNULL mutable_graph_def();
  void set_allocated_graph_def(::tensorflow::GraphDef* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_graph_def(::tensorflow::GraphDef* PROTOBUF_NULLABLE value);
  ::tensorflow::GraphDef* PROTOBUF_NULLABLE unsafe_arena_release_graph_def();

  private:
  const ::tensorflow::GraphDef& _internal_graph_def() const;
  ::tensorflow::GraphDef* PROTOBUF_NONNULL _internal_mutable_graph_def();

  public:
  // .tensorflow.SaverDef saver_def = 10;
  bool has_saver_def() const;
  void clear_saver_def() ;
  const ::tensorflow::SaverDef& saver_def() const;
  [[nodiscard]] ::tensorflow::SaverDef* PROTOBUF_NULLABLE release_saver_def();
  ::tensorflow::SaverDef* PROTOBUF_NONNULL mutable_saver_def();
  void set_allocated_saver_def(::tensorflow::SaverDef* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_saver_def(::tensorflow::SaverDef* PROTOBUF_NULLABLE value);
  ::tensorflow::SaverDef* PROTOBUF_NULLABLE unsafe_arena_release_saver_def();

  private:
  const ::tensorflow::SaverDef& _internal_saver_def() const;
  ::tensorflow::SaverDef* PROTOBUF_NONNULL _internal_mutable_saver_def();

  public:
  // @@protoc_insertion_point(class_scope:tensorflow.quantization.ExportedModel)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 6,
                                   4, 90,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExportedModel& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::MapField<ExportedModel_FunctionAliasesEntry_DoNotUse, std::string, std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        function_aliases_;
    ::google::protobuf::RepeatedPtrField< ::tensorflow::AssetFileDef > asset_file_defs_;
    ::google::protobuf::internal::ArenaStringPtr init_node_name_;
    ::google::protobuf::internal::ArenaStringPtr checkpoint_dir_;
    ::tensorflow::GraphDef* PROTOBUF_NULLABLE graph_def_;
    ::tensorflow::SaverDef* PROTOBUF_NULLABLE saver_def_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcompiler_2fmlir_2fquantization_2ftensorflow_2fexported_5fmodel_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull ExportedModel_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ExportedModel

// .tensorflow.GraphDef graph_def = 1;
inline bool ExportedModel::has_graph_def() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.graph_def_ != nullptr);
  return value;
}
inline const ::tensorflow::GraphDef& ExportedModel::_internal_graph_def() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::tensorflow::GraphDef* p = _impl_.graph_def_;
  return p != nullptr ? *p : reinterpret_cast<const ::tensorflow::GraphDef&>(::tensorflow::_GraphDef_default_instance_);
}
inline const ::tensorflow::GraphDef& ExportedModel::graph_def() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.quantization.ExportedModel.graph_def)
  return _internal_graph_def();
}
inline void ExportedModel::unsafe_arena_set_allocated_graph_def(
    ::tensorflow::GraphDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.graph_def_);
  }
  _impl_.graph_def_ = reinterpret_cast<::tensorflow::GraphDef*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:tensorflow.quantization.ExportedModel.graph_def)
}
inline ::tensorflow::GraphDef* PROTOBUF_NULLABLE ExportedModel::release_graph_def() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000004u;
  ::tensorflow::GraphDef* released = _impl_.graph_def_;
  _impl_.graph_def_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::tensorflow::GraphDef* PROTOBUF_NULLABLE ExportedModel::unsafe_arena_release_graph_def() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.quantization.ExportedModel.graph_def)

  _impl_._has_bits_[0] &= ~0x00000004u;
  ::tensorflow::GraphDef* temp = _impl_.graph_def_;
  _impl_.graph_def_ = nullptr;
  return temp;
}
inline ::tensorflow::GraphDef* PROTOBUF_NONNULL ExportedModel::_internal_mutable_graph_def() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.graph_def_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::tensorflow::GraphDef>(GetArena());
    _impl_.graph_def_ = reinterpret_cast<::tensorflow::GraphDef*>(p);
  }
  return _impl_.graph_def_;
}
inline ::tensorflow::GraphDef* PROTOBUF_NONNULL ExportedModel::mutable_graph_def()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000004u;
  ::tensorflow::GraphDef* _msg = _internal_mutable_graph_def();
  // @@protoc_insertion_point(field_mutable:tensorflow.quantization.ExportedModel.graph_def)
  return _msg;
}
inline void ExportedModel::set_allocated_graph_def(::tensorflow::GraphDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.graph_def_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }

  _impl_.graph_def_ = reinterpret_cast<::tensorflow::GraphDef*>(value);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.quantization.ExportedModel.graph_def)
}

// string init_node_name = 2;
inline void ExportedModel::clear_init_node_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.init_node_name_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::std::string& ExportedModel::init_node_name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.quantization.ExportedModel.init_node_name)
  return _internal_init_node_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExportedModel::set_init_node_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.init_node_name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:tensorflow.quantization.ExportedModel.init_node_name)
}
inline ::std::string* PROTOBUF_NONNULL ExportedModel::mutable_init_node_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_init_node_name();
  // @@protoc_insertion_point(field_mutable:tensorflow.quantization.ExportedModel.init_node_name)
  return _s;
}
inline const ::std::string& ExportedModel::_internal_init_node_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.init_node_name_.Get();
}
inline void ExportedModel::_internal_set_init_node_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.init_node_name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExportedModel::_internal_mutable_init_node_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.init_node_name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExportedModel::release_init_node_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.quantization.ExportedModel.init_node_name)
  if ((_impl_._has_bits_[0] & 0x00000001u) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* released = _impl_.init_node_name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.init_node_name_.Set("", GetArena());
  }
  return released;
}
inline void ExportedModel::set_allocated_init_node_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.init_node_name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.init_node_name_.IsDefault()) {
    _impl_.init_node_name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:tensorflow.quantization.ExportedModel.init_node_name)
}

// string checkpoint_dir = 5;
inline void ExportedModel::clear_checkpoint_dir() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.checkpoint_dir_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline const ::std::string& ExportedModel::checkpoint_dir() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.quantization.ExportedModel.checkpoint_dir)
  return _internal_checkpoint_dir();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExportedModel::set_checkpoint_dir(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.checkpoint_dir_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:tensorflow.quantization.ExportedModel.checkpoint_dir)
}
inline ::std::string* PROTOBUF_NONNULL ExportedModel::mutable_checkpoint_dir()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_checkpoint_dir();
  // @@protoc_insertion_point(field_mutable:tensorflow.quantization.ExportedModel.checkpoint_dir)
  return _s;
}
inline const ::std::string& ExportedModel::_internal_checkpoint_dir() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.checkpoint_dir_.Get();
}
inline void ExportedModel::_internal_set_checkpoint_dir(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.checkpoint_dir_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExportedModel::_internal_mutable_checkpoint_dir() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000002u;
  return _impl_.checkpoint_dir_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExportedModel::release_checkpoint_dir() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.quantization.ExportedModel.checkpoint_dir)
  if ((_impl_._has_bits_[0] & 0x00000002u) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000002u;
  auto* released = _impl_.checkpoint_dir_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.checkpoint_dir_.Set("", GetArena());
  }
  return released;
}
inline void ExportedModel::set_allocated_checkpoint_dir(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  _impl_.checkpoint_dir_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.checkpoint_dir_.IsDefault()) {
    _impl_.checkpoint_dir_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:tensorflow.quantization.ExportedModel.checkpoint_dir)
}

// map<string, string> function_aliases = 6;
inline int ExportedModel::_internal_function_aliases_size() const {
  return _internal_function_aliases().size();
}
inline int ExportedModel::function_aliases_size() const {
  return _internal_function_aliases_size();
}
inline void ExportedModel::clear_function_aliases() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.function_aliases_.Clear();
}
inline const ::google::protobuf::Map<std::string, std::string>& ExportedModel::_internal_function_aliases() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.function_aliases_.GetMap();
}
inline const ::google::protobuf::Map<std::string, std::string>& ExportedModel::function_aliases() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:tensorflow.quantization.ExportedModel.function_aliases)
  return _internal_function_aliases();
}
inline ::google::protobuf::Map<std::string, std::string>* PROTOBUF_NONNULL ExportedModel::_internal_mutable_function_aliases() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.function_aliases_.MutableMap();
}
inline ::google::protobuf::Map<std::string, std::string>* PROTOBUF_NONNULL ExportedModel::mutable_function_aliases()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_map:tensorflow.quantization.ExportedModel.function_aliases)
  return _internal_mutable_function_aliases();
}

// repeated .tensorflow.AssetFileDef asset_file_defs = 8;
inline int ExportedModel::_internal_asset_file_defs_size() const {
  return _internal_asset_file_defs().size();
}
inline int ExportedModel::asset_file_defs_size() const {
  return _internal_asset_file_defs_size();
}
inline ::tensorflow::AssetFileDef* PROTOBUF_NONNULL ExportedModel::mutable_asset_file_defs(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:tensorflow.quantization.ExportedModel.asset_file_defs)
  return _internal_mutable_asset_file_defs()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::tensorflow::AssetFileDef>* PROTOBUF_NONNULL ExportedModel::mutable_asset_file_defs()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:tensorflow.quantization.ExportedModel.asset_file_defs)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_asset_file_defs();
}
inline const ::tensorflow::AssetFileDef& ExportedModel::asset_file_defs(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.quantization.ExportedModel.asset_file_defs)
  return _internal_asset_file_defs().Get(index);
}
inline ::tensorflow::AssetFileDef* PROTOBUF_NONNULL ExportedModel::add_asset_file_defs()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::tensorflow::AssetFileDef* _add = _internal_mutable_asset_file_defs()->Add();
  // @@protoc_insertion_point(field_add:tensorflow.quantization.ExportedModel.asset_file_defs)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::tensorflow::AssetFileDef>& ExportedModel::asset_file_defs() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:tensorflow.quantization.ExportedModel.asset_file_defs)
  return _internal_asset_file_defs();
}
inline const ::google::protobuf::RepeatedPtrField<::tensorflow::AssetFileDef>&
ExportedModel::_internal_asset_file_defs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.asset_file_defs_;
}
inline ::google::protobuf::RepeatedPtrField<::tensorflow::AssetFileDef>* PROTOBUF_NONNULL
ExportedModel::_internal_mutable_asset_file_defs() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.asset_file_defs_;
}

// .tensorflow.SaverDef saver_def = 10;
inline bool ExportedModel::has_saver_def() const {
  bool value = (_impl_._has_bits_[0] & 0x00000008u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.saver_def_ != nullptr);
  return value;
}
inline const ::tensorflow::SaverDef& ExportedModel::_internal_saver_def() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::tensorflow::SaverDef* p = _impl_.saver_def_;
  return p != nullptr ? *p : reinterpret_cast<const ::tensorflow::SaverDef&>(::tensorflow::_SaverDef_default_instance_);
}
inline const ::tensorflow::SaverDef& ExportedModel::saver_def() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.quantization.ExportedModel.saver_def)
  return _internal_saver_def();
}
inline void ExportedModel::unsafe_arena_set_allocated_saver_def(
    ::tensorflow::SaverDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.saver_def_);
  }
  _impl_.saver_def_ = reinterpret_cast<::tensorflow::SaverDef*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000008u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000008u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:tensorflow.quantization.ExportedModel.saver_def)
}
inline ::tensorflow::SaverDef* PROTOBUF_NULLABLE ExportedModel::release_saver_def() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000008u;
  ::tensorflow::SaverDef* released = _impl_.saver_def_;
  _impl_.saver_def_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::tensorflow::SaverDef* PROTOBUF_NULLABLE ExportedModel::unsafe_arena_release_saver_def() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.quantization.ExportedModel.saver_def)

  _impl_._has_bits_[0] &= ~0x00000008u;
  ::tensorflow::SaverDef* temp = _impl_.saver_def_;
  _impl_.saver_def_ = nullptr;
  return temp;
}
inline ::tensorflow::SaverDef* PROTOBUF_NONNULL ExportedModel::_internal_mutable_saver_def() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.saver_def_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::tensorflow::SaverDef>(GetArena());
    _impl_.saver_def_ = reinterpret_cast<::tensorflow::SaverDef*>(p);
  }
  return _impl_.saver_def_;
}
inline ::tensorflow::SaverDef* PROTOBUF_NONNULL ExportedModel::mutable_saver_def()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000008u;
  ::tensorflow::SaverDef* _msg = _internal_mutable_saver_def();
  // @@protoc_insertion_point(field_mutable:tensorflow.quantization.ExportedModel.saver_def)
  return _msg;
}
inline void ExportedModel::set_allocated_saver_def(::tensorflow::SaverDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.saver_def_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000008u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000008u;
  }

  _impl_.saver_def_ = reinterpret_cast<::tensorflow::SaverDef*>(value);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.quantization.ExportedModel.saver_def)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace quantization
}  // namespace tensorflow


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // tensorflow_2fcompiler_2fmlir_2fquantization_2ftensorflow_2fexported_5fmodel_2eproto_2epb_2eh
