
#ifndef JXL_CMS_EXPORT_H
#define JXL_CMS_EXPORT_H

#define JXL_CMS_EXPORT
#define JXL_CMS_NO_EXPORT

#ifndef JXL_CMS_DEPRECATED
#  define JXL_CMS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#endif

