
/* Autogenerated by mlir-tblgen; don't manually edit. */

#include "mlir-c/Pass.h"

#ifdef __cplusplus
extern "C" {
#endif

// Registration for the entire group
MLIR_CAPI_EXPORTED void mlirRegisterPasses(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateMathExpandOpsPass(void);
MLIR_CAPI_EXPORTED void mlirRegisterMathExpandOpsPass(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateMathExtendToSupportedTypes(void);
MLIR_CAPI_EXPORTED void mlirRegisterMathExtendToSupportedTypes(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateMathSincosFusionPass(void);
MLIR_CAPI_EXPORTED void mlirRegisterMathSincosFusionPass(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateMathUpliftToFMA(void);
MLIR_CAPI_EXPORTED void mlirRegisterMathUpliftToFMA(void);



#ifdef __cplusplus
}
#endif
