// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: tensorflow/core/debug/debugger_event_metadata.proto
// Protobuf C++ Version: 6.31.1

#ifndef tensorflow_2fcore_2fdebug_2fdebugger_5fevent_5fmetadata_2eproto_2epb_2eh
#define tensorflow_2fcore_2fdebug_2fdebugger_5fevent_5fmetadata_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6031001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_tensorflow_2fcore_2fdebug_2fdebugger_5fevent_5fmetadata_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_tensorflow_2fcore_2fdebug_2fdebugger_5fevent_5fmetadata_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable descriptor_table_tensorflow_2fcore_2fdebug_2fdebugger_5fevent_5fmetadata_2eproto;
}  // extern "C"
namespace third_party {
namespace tensorflow {
namespace core {
namespace debug {
class DebuggerEventMetadata;
struct DebuggerEventMetadataDefaultTypeInternal;
PROTOBUF_EXPORT extern DebuggerEventMetadataDefaultTypeInternal _DebuggerEventMetadata_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull DebuggerEventMetadata_class_data_;
}  // namespace debug
}  // namespace core
}  // namespace tensorflow
}  // namespace third_party
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace third_party {
namespace tensorflow {
namespace core {
namespace debug {

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT DebuggerEventMetadata final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:third_party.tensorflow.core.debug.DebuggerEventMetadata) */ {
 public:
  inline DebuggerEventMetadata() : DebuggerEventMetadata(nullptr) {}
  ~DebuggerEventMetadata() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(DebuggerEventMetadata* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(DebuggerEventMetadata));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR DebuggerEventMetadata(::google::protobuf::internal::ConstantInitialized);

  inline DebuggerEventMetadata(const DebuggerEventMetadata& from) : DebuggerEventMetadata(nullptr, from) {}
  inline DebuggerEventMetadata(DebuggerEventMetadata&& from) noexcept
      : DebuggerEventMetadata(nullptr, ::std::move(from)) {}
  inline DebuggerEventMetadata& operator=(const DebuggerEventMetadata& from) {
    CopyFrom(from);
    return *this;
  }
  inline DebuggerEventMetadata& operator=(DebuggerEventMetadata&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const DebuggerEventMetadata& default_instance() {
    return *reinterpret_cast<const DebuggerEventMetadata*>(
        &_DebuggerEventMetadata_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(DebuggerEventMetadata& a, DebuggerEventMetadata& b) { a.Swap(&b); }
  inline void Swap(DebuggerEventMetadata* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DebuggerEventMetadata* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  DebuggerEventMetadata* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<DebuggerEventMetadata>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const DebuggerEventMetadata& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const DebuggerEventMetadata& from) { DebuggerEventMetadata::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(DebuggerEventMetadata* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "third_party.tensorflow.core.debug.DebuggerEventMetadata"; }

 protected:
  explicit DebuggerEventMetadata(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  DebuggerEventMetadata(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DebuggerEventMetadata& from);
  DebuggerEventMetadata(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, DebuggerEventMetadata&& from) noexcept
      : DebuggerEventMetadata(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kDeviceFieldNumber = 1,
    kOutputSlotFieldNumber = 2,
    kNumChunksFieldNumber = 3,
    kChunkIndexFieldNumber = 4,
  };
  // string device = 1;
  void clear_device() ;
  const ::std::string& device() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_device(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_device();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_device();
  void set_allocated_device(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_device() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_device(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_device();

  public:
  // int32 output_slot = 2;
  void clear_output_slot() ;
  ::int32_t output_slot() const;
  void set_output_slot(::int32_t value);

  private:
  ::int32_t _internal_output_slot() const;
  void _internal_set_output_slot(::int32_t value);

  public:
  // int32 num_chunks = 3;
  void clear_num_chunks() ;
  ::int32_t num_chunks() const;
  void set_num_chunks(::int32_t value);

  private:
  ::int32_t _internal_num_chunks() const;
  void _internal_set_num_chunks(::int32_t value);

  public:
  // int32 chunk_index = 4;
  void clear_chunk_index() ;
  ::int32_t chunk_index() const;
  void set_chunk_index(::int32_t value);

  private:
  ::int32_t _internal_chunk_index() const;
  void _internal_set_chunk_index(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:third_party.tensorflow.core.debug.DebuggerEventMetadata)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   0, 70,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const DebuggerEventMetadata& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr device_;
    ::int32_t output_slot_;
    ::int32_t num_chunks_;
    ::int32_t chunk_index_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcore_2fdebug_2fdebugger_5fevent_5fmetadata_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull DebuggerEventMetadata_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// DebuggerEventMetadata

// string device = 1;
inline void DebuggerEventMetadata::clear_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.device_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::std::string& DebuggerEventMetadata::device() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:third_party.tensorflow.core.debug.DebuggerEventMetadata.device)
  return _internal_device();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void DebuggerEventMetadata::set_device(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.device_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:third_party.tensorflow.core.debug.DebuggerEventMetadata.device)
}
inline ::std::string* PROTOBUF_NONNULL DebuggerEventMetadata::mutable_device()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_device();
  // @@protoc_insertion_point(field_mutable:third_party.tensorflow.core.debug.DebuggerEventMetadata.device)
  return _s;
}
inline const ::std::string& DebuggerEventMetadata::_internal_device() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.device_.Get();
}
inline void DebuggerEventMetadata::_internal_set_device(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.device_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL DebuggerEventMetadata::_internal_mutable_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.device_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE DebuggerEventMetadata::release_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:third_party.tensorflow.core.debug.DebuggerEventMetadata.device)
  if ((_impl_._has_bits_[0] & 0x00000001u) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* released = _impl_.device_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.device_.Set("", GetArena());
  }
  return released;
}
inline void DebuggerEventMetadata::set_allocated_device(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.device_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.device_.IsDefault()) {
    _impl_.device_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:third_party.tensorflow.core.debug.DebuggerEventMetadata.device)
}

// int32 output_slot = 2;
inline void DebuggerEventMetadata::clear_output_slot() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.output_slot_ = 0;
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline ::int32_t DebuggerEventMetadata::output_slot() const {
  // @@protoc_insertion_point(field_get:third_party.tensorflow.core.debug.DebuggerEventMetadata.output_slot)
  return _internal_output_slot();
}
inline void DebuggerEventMetadata::set_output_slot(::int32_t value) {
  _internal_set_output_slot(value);
  _impl_._has_bits_[0] |= 0x00000002u;
  // @@protoc_insertion_point(field_set:third_party.tensorflow.core.debug.DebuggerEventMetadata.output_slot)
}
inline ::int32_t DebuggerEventMetadata::_internal_output_slot() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.output_slot_;
}
inline void DebuggerEventMetadata::_internal_set_output_slot(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.output_slot_ = value;
}

// int32 num_chunks = 3;
inline void DebuggerEventMetadata::clear_num_chunks() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.num_chunks_ = 0;
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline ::int32_t DebuggerEventMetadata::num_chunks() const {
  // @@protoc_insertion_point(field_get:third_party.tensorflow.core.debug.DebuggerEventMetadata.num_chunks)
  return _internal_num_chunks();
}
inline void DebuggerEventMetadata::set_num_chunks(::int32_t value) {
  _internal_set_num_chunks(value);
  _impl_._has_bits_[0] |= 0x00000004u;
  // @@protoc_insertion_point(field_set:third_party.tensorflow.core.debug.DebuggerEventMetadata.num_chunks)
}
inline ::int32_t DebuggerEventMetadata::_internal_num_chunks() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.num_chunks_;
}
inline void DebuggerEventMetadata::_internal_set_num_chunks(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.num_chunks_ = value;
}

// int32 chunk_index = 4;
inline void DebuggerEventMetadata::clear_chunk_index() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.chunk_index_ = 0;
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline ::int32_t DebuggerEventMetadata::chunk_index() const {
  // @@protoc_insertion_point(field_get:third_party.tensorflow.core.debug.DebuggerEventMetadata.chunk_index)
  return _internal_chunk_index();
}
inline void DebuggerEventMetadata::set_chunk_index(::int32_t value) {
  _internal_set_chunk_index(value);
  _impl_._has_bits_[0] |= 0x00000008u;
  // @@protoc_insertion_point(field_set:third_party.tensorflow.core.debug.DebuggerEventMetadata.chunk_index)
}
inline ::int32_t DebuggerEventMetadata::_internal_chunk_index() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.chunk_index_;
}
inline void DebuggerEventMetadata::_internal_set_chunk_index(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.chunk_index_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace debug
}  // namespace core
}  // namespace tensorflow
}  // namespace third_party


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // tensorflow_2fcore_2fdebug_2fdebugger_5fevent_5fmetadata_2eproto_2epb_2eh
