// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: xla/stream_executor/gpu/tma_metadata.proto
// Protobuf C++ Version: 6.31.1

#ifndef xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto_2epb_2eh
#define xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6031001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/map.h"  // IWYU pragma: export
#include "google/protobuf/map_type_handler.h"  // IWYU pragma: export
#include "google/protobuf/map_entry.h"
#include "google/protobuf/map_field.h"
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable descriptor_table_xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto;
}  // extern "C"
namespace stream_executor {
namespace gpu {
enum TmaDescriptorProto_TmaFloatOobFill : int;
PROTOBUF_EXPORT extern const uint32_t TmaDescriptorProto_TmaFloatOobFill_internal_data_[];
enum TmaDescriptorProto_TmaInterleave : int;
PROTOBUF_EXPORT extern const uint32_t TmaDescriptorProto_TmaInterleave_internal_data_[];
enum TmaDescriptorProto_TmaL2Promotion : int;
PROTOBUF_EXPORT extern const uint32_t TmaDescriptorProto_TmaL2Promotion_internal_data_[];
enum TmaDescriptorProto_TmaSwizzle : int;
PROTOBUF_EXPORT extern const uint32_t TmaDescriptorProto_TmaSwizzle_internal_data_[];
class TmaDescriptorProto;
struct TmaDescriptorProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern TmaDescriptorProtoDefaultTypeInternal _TmaDescriptorProto_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull TmaDescriptorProto_class_data_;
class TmaMetadataProto;
struct TmaMetadataProtoDefaultTypeInternal;
PROTOBUF_EXPORT extern TmaMetadataProtoDefaultTypeInternal _TmaMetadataProto_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull TmaMetadataProto_class_data_;
class TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse;
struct TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUseDefaultTypeInternal;
PROTOBUF_EXPORT extern TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUseDefaultTypeInternal _TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse_class_data_;
}  // namespace gpu
}  // namespace stream_executor
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill_internal_data_>
    internal::EnumTraitsImpl::value<::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill>;
template <>
internal::EnumTraitsT<::stream_executor::gpu::TmaDescriptorProto_TmaInterleave_internal_data_>
    internal::EnumTraitsImpl::value<::stream_executor::gpu::TmaDescriptorProto_TmaInterleave>;
template <>
internal::EnumTraitsT<::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion_internal_data_>
    internal::EnumTraitsImpl::value<::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion>;
template <>
internal::EnumTraitsT<::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle_internal_data_>
    internal::EnumTraitsImpl::value<::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle>;
}  // namespace protobuf
}  // namespace google

namespace stream_executor {
namespace gpu {
enum TmaDescriptorProto_TmaInterleave : int {
  TmaDescriptorProto_TmaInterleave_INTERLEAVE_NONE = 0,
  TmaDescriptorProto_TmaInterleave_INTERLEAVE_BYTES16 = 1,
  TmaDescriptorProto_TmaInterleave_INTERLEAVE_BYTES32 = 2,
  TmaDescriptorProto_TmaInterleave_TmaDescriptorProto_TmaInterleave_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  TmaDescriptorProto_TmaInterleave_TmaDescriptorProto_TmaInterleave_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

PROTOBUF_EXPORT extern const uint32_t TmaDescriptorProto_TmaInterleave_internal_data_[];
inline constexpr TmaDescriptorProto_TmaInterleave TmaDescriptorProto_TmaInterleave_TmaInterleave_MIN =
    static_cast<TmaDescriptorProto_TmaInterleave>(0);
inline constexpr TmaDescriptorProto_TmaInterleave TmaDescriptorProto_TmaInterleave_TmaInterleave_MAX =
    static_cast<TmaDescriptorProto_TmaInterleave>(2);
inline bool TmaDescriptorProto_TmaInterleave_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int TmaDescriptorProto_TmaInterleave_TmaInterleave_ARRAYSIZE = 2 + 1;
PROTOBUF_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL TmaDescriptorProto_TmaInterleave_descriptor();
template <typename T>
const ::std::string& TmaDescriptorProto_TmaInterleave_Name(T value) {
  static_assert(::std::is_same<T, TmaDescriptorProto_TmaInterleave>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to TmaInterleave_Name().");
  return TmaDescriptorProto_TmaInterleave_Name(static_cast<TmaDescriptorProto_TmaInterleave>(value));
}
template <>
inline const ::std::string& TmaDescriptorProto_TmaInterleave_Name(TmaDescriptorProto_TmaInterleave value) {
  return ::google::protobuf::internal::NameOfDenseEnum<TmaDescriptorProto_TmaInterleave_descriptor, 0, 2>(
      static_cast<int>(value));
}
inline bool TmaDescriptorProto_TmaInterleave_Parse(
    ::absl::string_view name, TmaDescriptorProto_TmaInterleave* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<TmaDescriptorProto_TmaInterleave>(TmaDescriptorProto_TmaInterleave_descriptor(), name,
                                           value);
}
enum TmaDescriptorProto_TmaSwizzle : int {
  TmaDescriptorProto_TmaSwizzle_SWIZZLE_NONE = 0,
  TmaDescriptorProto_TmaSwizzle_SWIZZLE_BYTES32 = 1,
  TmaDescriptorProto_TmaSwizzle_SWIZZLE_BYTES64 = 2,
  TmaDescriptorProto_TmaSwizzle_SWIZZLE_BYTES128 = 3,
  TmaDescriptorProto_TmaSwizzle_TmaDescriptorProto_TmaSwizzle_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  TmaDescriptorProto_TmaSwizzle_TmaDescriptorProto_TmaSwizzle_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

PROTOBUF_EXPORT extern const uint32_t TmaDescriptorProto_TmaSwizzle_internal_data_[];
inline constexpr TmaDescriptorProto_TmaSwizzle TmaDescriptorProto_TmaSwizzle_TmaSwizzle_MIN =
    static_cast<TmaDescriptorProto_TmaSwizzle>(0);
inline constexpr TmaDescriptorProto_TmaSwizzle TmaDescriptorProto_TmaSwizzle_TmaSwizzle_MAX =
    static_cast<TmaDescriptorProto_TmaSwizzle>(3);
inline bool TmaDescriptorProto_TmaSwizzle_IsValid(int value) {
  return 0 <= value && value <= 3;
}
inline constexpr int TmaDescriptorProto_TmaSwizzle_TmaSwizzle_ARRAYSIZE = 3 + 1;
PROTOBUF_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL TmaDescriptorProto_TmaSwizzle_descriptor();
template <typename T>
const ::std::string& TmaDescriptorProto_TmaSwizzle_Name(T value) {
  static_assert(::std::is_same<T, TmaDescriptorProto_TmaSwizzle>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to TmaSwizzle_Name().");
  return TmaDescriptorProto_TmaSwizzle_Name(static_cast<TmaDescriptorProto_TmaSwizzle>(value));
}
template <>
inline const ::std::string& TmaDescriptorProto_TmaSwizzle_Name(TmaDescriptorProto_TmaSwizzle value) {
  return ::google::protobuf::internal::NameOfDenseEnum<TmaDescriptorProto_TmaSwizzle_descriptor, 0, 3>(
      static_cast<int>(value));
}
inline bool TmaDescriptorProto_TmaSwizzle_Parse(
    ::absl::string_view name, TmaDescriptorProto_TmaSwizzle* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<TmaDescriptorProto_TmaSwizzle>(TmaDescriptorProto_TmaSwizzle_descriptor(), name,
                                           value);
}
enum TmaDescriptorProto_TmaL2Promotion : int {
  TmaDescriptorProto_TmaL2Promotion_L2_PROMOTION_NONE = 0,
  TmaDescriptorProto_TmaL2Promotion_L2_PROMOTION_BYTES64 = 1,
  TmaDescriptorProto_TmaL2Promotion_L2_PROMOTION_BYTES128 = 2,
  TmaDescriptorProto_TmaL2Promotion_L2_PROMOTION_BYTES256 = 3,
  TmaDescriptorProto_TmaL2Promotion_TmaDescriptorProto_TmaL2Promotion_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  TmaDescriptorProto_TmaL2Promotion_TmaDescriptorProto_TmaL2Promotion_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

PROTOBUF_EXPORT extern const uint32_t TmaDescriptorProto_TmaL2Promotion_internal_data_[];
inline constexpr TmaDescriptorProto_TmaL2Promotion TmaDescriptorProto_TmaL2Promotion_TmaL2Promotion_MIN =
    static_cast<TmaDescriptorProto_TmaL2Promotion>(0);
inline constexpr TmaDescriptorProto_TmaL2Promotion TmaDescriptorProto_TmaL2Promotion_TmaL2Promotion_MAX =
    static_cast<TmaDescriptorProto_TmaL2Promotion>(3);
inline bool TmaDescriptorProto_TmaL2Promotion_IsValid(int value) {
  return 0 <= value && value <= 3;
}
inline constexpr int TmaDescriptorProto_TmaL2Promotion_TmaL2Promotion_ARRAYSIZE = 3 + 1;
PROTOBUF_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL TmaDescriptorProto_TmaL2Promotion_descriptor();
template <typename T>
const ::std::string& TmaDescriptorProto_TmaL2Promotion_Name(T value) {
  static_assert(::std::is_same<T, TmaDescriptorProto_TmaL2Promotion>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to TmaL2Promotion_Name().");
  return TmaDescriptorProto_TmaL2Promotion_Name(static_cast<TmaDescriptorProto_TmaL2Promotion>(value));
}
template <>
inline const ::std::string& TmaDescriptorProto_TmaL2Promotion_Name(TmaDescriptorProto_TmaL2Promotion value) {
  return ::google::protobuf::internal::NameOfDenseEnum<TmaDescriptorProto_TmaL2Promotion_descriptor, 0, 3>(
      static_cast<int>(value));
}
inline bool TmaDescriptorProto_TmaL2Promotion_Parse(
    ::absl::string_view name, TmaDescriptorProto_TmaL2Promotion* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<TmaDescriptorProto_TmaL2Promotion>(TmaDescriptorProto_TmaL2Promotion_descriptor(), name,
                                           value);
}
enum TmaDescriptorProto_TmaFloatOobFill : int {
  TmaDescriptorProto_TmaFloatOobFill_FLOAT_OOB_FILL_NONE = 0,
  TmaDescriptorProto_TmaFloatOobFill_FLOAT_OOB_FILL_NAN_REQUEST_ZERO_FMA = 1,
  TmaDescriptorProto_TmaFloatOobFill_TmaDescriptorProto_TmaFloatOobFill_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  TmaDescriptorProto_TmaFloatOobFill_TmaDescriptorProto_TmaFloatOobFill_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

PROTOBUF_EXPORT extern const uint32_t TmaDescriptorProto_TmaFloatOobFill_internal_data_[];
inline constexpr TmaDescriptorProto_TmaFloatOobFill TmaDescriptorProto_TmaFloatOobFill_TmaFloatOobFill_MIN =
    static_cast<TmaDescriptorProto_TmaFloatOobFill>(0);
inline constexpr TmaDescriptorProto_TmaFloatOobFill TmaDescriptorProto_TmaFloatOobFill_TmaFloatOobFill_MAX =
    static_cast<TmaDescriptorProto_TmaFloatOobFill>(1);
inline bool TmaDescriptorProto_TmaFloatOobFill_IsValid(int value) {
  return 0 <= value && value <= 1;
}
inline constexpr int TmaDescriptorProto_TmaFloatOobFill_TmaFloatOobFill_ARRAYSIZE = 1 + 1;
PROTOBUF_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL TmaDescriptorProto_TmaFloatOobFill_descriptor();
template <typename T>
const ::std::string& TmaDescriptorProto_TmaFloatOobFill_Name(T value) {
  static_assert(::std::is_same<T, TmaDescriptorProto_TmaFloatOobFill>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to TmaFloatOobFill_Name().");
  return TmaDescriptorProto_TmaFloatOobFill_Name(static_cast<TmaDescriptorProto_TmaFloatOobFill>(value));
}
template <>
inline const ::std::string& TmaDescriptorProto_TmaFloatOobFill_Name(TmaDescriptorProto_TmaFloatOobFill value) {
  return ::google::protobuf::internal::NameOfDenseEnum<TmaDescriptorProto_TmaFloatOobFill_descriptor, 0, 1>(
      static_cast<int>(value));
}
inline bool TmaDescriptorProto_TmaFloatOobFill_Parse(
    ::absl::string_view name, TmaDescriptorProto_TmaFloatOobFill* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<TmaDescriptorProto_TmaFloatOobFill>(TmaDescriptorProto_TmaFloatOobFill_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT TmaDescriptorProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:stream_executor.gpu.TmaDescriptorProto) */ {
 public:
  inline TmaDescriptorProto() : TmaDescriptorProto(nullptr) {}
  ~TmaDescriptorProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TmaDescriptorProto* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TmaDescriptorProto));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR TmaDescriptorProto(::google::protobuf::internal::ConstantInitialized);

  inline TmaDescriptorProto(const TmaDescriptorProto& from) : TmaDescriptorProto(nullptr, from) {}
  inline TmaDescriptorProto(TmaDescriptorProto&& from) noexcept
      : TmaDescriptorProto(nullptr, ::std::move(from)) {}
  inline TmaDescriptorProto& operator=(const TmaDescriptorProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline TmaDescriptorProto& operator=(TmaDescriptorProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const TmaDescriptorProto& default_instance() {
    return *reinterpret_cast<const TmaDescriptorProto*>(
        &_TmaDescriptorProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(TmaDescriptorProto& a, TmaDescriptorProto& b) { a.Swap(&b); }
  inline void Swap(TmaDescriptorProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TmaDescriptorProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  TmaDescriptorProto* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<TmaDescriptorProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const TmaDescriptorProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const TmaDescriptorProto& from) { TmaDescriptorProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TmaDescriptorProto* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "stream_executor.gpu.TmaDescriptorProto"; }

 protected:
  explicit TmaDescriptorProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TmaDescriptorProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TmaDescriptorProto& from);
  TmaDescriptorProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TmaDescriptorProto&& from) noexcept
      : TmaDescriptorProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using TmaInterleave = TmaDescriptorProto_TmaInterleave;
  static constexpr TmaInterleave INTERLEAVE_NONE = TmaDescriptorProto_TmaInterleave_INTERLEAVE_NONE;
  static constexpr TmaInterleave INTERLEAVE_BYTES16 = TmaDescriptorProto_TmaInterleave_INTERLEAVE_BYTES16;
  static constexpr TmaInterleave INTERLEAVE_BYTES32 = TmaDescriptorProto_TmaInterleave_INTERLEAVE_BYTES32;
  static inline bool TmaInterleave_IsValid(int value) {
    return TmaDescriptorProto_TmaInterleave_IsValid(value);
  }
  static constexpr TmaInterleave TmaInterleave_MIN = TmaDescriptorProto_TmaInterleave_TmaInterleave_MIN;
  static constexpr TmaInterleave TmaInterleave_MAX = TmaDescriptorProto_TmaInterleave_TmaInterleave_MAX;
  static constexpr int TmaInterleave_ARRAYSIZE = TmaDescriptorProto_TmaInterleave_TmaInterleave_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL TmaInterleave_descriptor() {
    return TmaDescriptorProto_TmaInterleave_descriptor();
  }
  template <typename T>
  static inline const ::std::string& TmaInterleave_Name(T value) {
    return TmaDescriptorProto_TmaInterleave_Name(value);
  }
  static inline bool TmaInterleave_Parse(
      ::absl::string_view name, TmaInterleave* PROTOBUF_NONNULL value) {
    return TmaDescriptorProto_TmaInterleave_Parse(name, value);
  }
  using TmaSwizzle = TmaDescriptorProto_TmaSwizzle;
  static constexpr TmaSwizzle SWIZZLE_NONE = TmaDescriptorProto_TmaSwizzle_SWIZZLE_NONE;
  static constexpr TmaSwizzle SWIZZLE_BYTES32 = TmaDescriptorProto_TmaSwizzle_SWIZZLE_BYTES32;
  static constexpr TmaSwizzle SWIZZLE_BYTES64 = TmaDescriptorProto_TmaSwizzle_SWIZZLE_BYTES64;
  static constexpr TmaSwizzle SWIZZLE_BYTES128 = TmaDescriptorProto_TmaSwizzle_SWIZZLE_BYTES128;
  static inline bool TmaSwizzle_IsValid(int value) {
    return TmaDescriptorProto_TmaSwizzle_IsValid(value);
  }
  static constexpr TmaSwizzle TmaSwizzle_MIN = TmaDescriptorProto_TmaSwizzle_TmaSwizzle_MIN;
  static constexpr TmaSwizzle TmaSwizzle_MAX = TmaDescriptorProto_TmaSwizzle_TmaSwizzle_MAX;
  static constexpr int TmaSwizzle_ARRAYSIZE = TmaDescriptorProto_TmaSwizzle_TmaSwizzle_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL TmaSwizzle_descriptor() {
    return TmaDescriptorProto_TmaSwizzle_descriptor();
  }
  template <typename T>
  static inline const ::std::string& TmaSwizzle_Name(T value) {
    return TmaDescriptorProto_TmaSwizzle_Name(value);
  }
  static inline bool TmaSwizzle_Parse(
      ::absl::string_view name, TmaSwizzle* PROTOBUF_NONNULL value) {
    return TmaDescriptorProto_TmaSwizzle_Parse(name, value);
  }
  using TmaL2Promotion = TmaDescriptorProto_TmaL2Promotion;
  static constexpr TmaL2Promotion L2_PROMOTION_NONE = TmaDescriptorProto_TmaL2Promotion_L2_PROMOTION_NONE;
  static constexpr TmaL2Promotion L2_PROMOTION_BYTES64 = TmaDescriptorProto_TmaL2Promotion_L2_PROMOTION_BYTES64;
  static constexpr TmaL2Promotion L2_PROMOTION_BYTES128 = TmaDescriptorProto_TmaL2Promotion_L2_PROMOTION_BYTES128;
  static constexpr TmaL2Promotion L2_PROMOTION_BYTES256 = TmaDescriptorProto_TmaL2Promotion_L2_PROMOTION_BYTES256;
  static inline bool TmaL2Promotion_IsValid(int value) {
    return TmaDescriptorProto_TmaL2Promotion_IsValid(value);
  }
  static constexpr TmaL2Promotion TmaL2Promotion_MIN = TmaDescriptorProto_TmaL2Promotion_TmaL2Promotion_MIN;
  static constexpr TmaL2Promotion TmaL2Promotion_MAX = TmaDescriptorProto_TmaL2Promotion_TmaL2Promotion_MAX;
  static constexpr int TmaL2Promotion_ARRAYSIZE = TmaDescriptorProto_TmaL2Promotion_TmaL2Promotion_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL TmaL2Promotion_descriptor() {
    return TmaDescriptorProto_TmaL2Promotion_descriptor();
  }
  template <typename T>
  static inline const ::std::string& TmaL2Promotion_Name(T value) {
    return TmaDescriptorProto_TmaL2Promotion_Name(value);
  }
  static inline bool TmaL2Promotion_Parse(
      ::absl::string_view name, TmaL2Promotion* PROTOBUF_NONNULL value) {
    return TmaDescriptorProto_TmaL2Promotion_Parse(name, value);
  }
  using TmaFloatOobFill = TmaDescriptorProto_TmaFloatOobFill;
  static constexpr TmaFloatOobFill FLOAT_OOB_FILL_NONE = TmaDescriptorProto_TmaFloatOobFill_FLOAT_OOB_FILL_NONE;
  static constexpr TmaFloatOobFill FLOAT_OOB_FILL_NAN_REQUEST_ZERO_FMA = TmaDescriptorProto_TmaFloatOobFill_FLOAT_OOB_FILL_NAN_REQUEST_ZERO_FMA;
  static inline bool TmaFloatOobFill_IsValid(int value) {
    return TmaDescriptorProto_TmaFloatOobFill_IsValid(value);
  }
  static constexpr TmaFloatOobFill TmaFloatOobFill_MIN = TmaDescriptorProto_TmaFloatOobFill_TmaFloatOobFill_MIN;
  static constexpr TmaFloatOobFill TmaFloatOobFill_MAX = TmaDescriptorProto_TmaFloatOobFill_TmaFloatOobFill_MAX;
  static constexpr int TmaFloatOobFill_ARRAYSIZE = TmaDescriptorProto_TmaFloatOobFill_TmaFloatOobFill_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL TmaFloatOobFill_descriptor() {
    return TmaDescriptorProto_TmaFloatOobFill_descriptor();
  }
  template <typename T>
  static inline const ::std::string& TmaFloatOobFill_Name(T value) {
    return TmaDescriptorProto_TmaFloatOobFill_Name(value);
  }
  static inline bool TmaFloatOobFill_Parse(
      ::absl::string_view name, TmaFloatOobFill* PROTOBUF_NONNULL value) {
    return TmaDescriptorProto_TmaFloatOobFill_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kGlobalDimsFieldNumber = 2,
    kGlobalStridesFieldNumber = 3,
    kBoxDimsFieldNumber = 4,
    kElementStridesFieldNumber = 5,
    kElementSizeFieldNumber = 1,
    kInterleaveFieldNumber = 6,
    kSwizzleFieldNumber = 7,
    kL2PromotionFieldNumber = 8,
    kFloatOobFillFieldNumber = 9,
  };
  // repeated int64 global_dims = 2;
  int global_dims_size() const;
  private:
  int _internal_global_dims_size() const;

  public:
  void clear_global_dims() ;
  ::int64_t global_dims(int index) const;
  void set_global_dims(int index, ::int64_t value);
  void add_global_dims(::int64_t value);
  const ::google::protobuf::RepeatedField<::int64_t>& global_dims() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_global_dims();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_global_dims() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_global_dims();

  public:
  // repeated int64 global_strides = 3;
  int global_strides_size() const;
  private:
  int _internal_global_strides_size() const;

  public:
  void clear_global_strides() ;
  ::int64_t global_strides(int index) const;
  void set_global_strides(int index, ::int64_t value);
  void add_global_strides(::int64_t value);
  const ::google::protobuf::RepeatedField<::int64_t>& global_strides() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_global_strides();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_global_strides() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_global_strides();

  public:
  // repeated int32 box_dims = 4;
  int box_dims_size() const;
  private:
  int _internal_box_dims_size() const;

  public:
  void clear_box_dims() ;
  ::int32_t box_dims(int index) const;
  void set_box_dims(int index, ::int32_t value);
  void add_box_dims(::int32_t value);
  const ::google::protobuf::RepeatedField<::int32_t>& box_dims() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_box_dims();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_box_dims() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_box_dims();

  public:
  // repeated int32 element_strides = 5;
  int element_strides_size() const;
  private:
  int _internal_element_strides_size() const;

  public:
  void clear_element_strides() ;
  ::int32_t element_strides(int index) const;
  void set_element_strides(int index, ::int32_t value);
  void add_element_strides(::int32_t value);
  const ::google::protobuf::RepeatedField<::int32_t>& element_strides() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_element_strides();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_element_strides() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_element_strides();

  public:
  // int64 element_size = 1;
  void clear_element_size() ;
  ::int64_t element_size() const;
  void set_element_size(::int64_t value);

  private:
  ::int64_t _internal_element_size() const;
  void _internal_set_element_size(::int64_t value);

  public:
  // .stream_executor.gpu.TmaDescriptorProto.TmaInterleave interleave = 6;
  void clear_interleave() ;
  ::stream_executor::gpu::TmaDescriptorProto_TmaInterleave interleave() const;
  void set_interleave(::stream_executor::gpu::TmaDescriptorProto_TmaInterleave value);

  private:
  ::stream_executor::gpu::TmaDescriptorProto_TmaInterleave _internal_interleave() const;
  void _internal_set_interleave(::stream_executor::gpu::TmaDescriptorProto_TmaInterleave value);

  public:
  // .stream_executor.gpu.TmaDescriptorProto.TmaSwizzle swizzle = 7;
  void clear_swizzle() ;
  ::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle swizzle() const;
  void set_swizzle(::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle value);

  private:
  ::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle _internal_swizzle() const;
  void _internal_set_swizzle(::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle value);

  public:
  // .stream_executor.gpu.TmaDescriptorProto.TmaL2Promotion l2_promotion = 8;
  void clear_l2_promotion() ;
  ::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion l2_promotion() const;
  void set_l2_promotion(::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion value);

  private:
  ::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion _internal_l2_promotion() const;
  void _internal_set_l2_promotion(::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion value);

  public:
  // .stream_executor.gpu.TmaDescriptorProto.TmaFloatOobFill float_oob_fill = 9;
  void clear_float_oob_fill() ;
  ::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill float_oob_fill() const;
  void set_float_oob_fill(::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill value);

  private:
  ::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill _internal_float_oob_fill() const;
  void _internal_set_float_oob_fill(::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill value);

  public:
  // @@protoc_insertion_point(class_scope:stream_executor.gpu.TmaDescriptorProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 9,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TmaDescriptorProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::int64_t> global_dims_;
    ::google::protobuf::internal::CachedSize _global_dims_cached_byte_size_;
    ::google::protobuf::RepeatedField<::int64_t> global_strides_;
    ::google::protobuf::internal::CachedSize _global_strides_cached_byte_size_;
    ::google::protobuf::RepeatedField<::int32_t> box_dims_;
    ::google::protobuf::internal::CachedSize _box_dims_cached_byte_size_;
    ::google::protobuf::RepeatedField<::int32_t> element_strides_;
    ::google::protobuf::internal::CachedSize _element_strides_cached_byte_size_;
    ::int64_t element_size_;
    int interleave_;
    int swizzle_;
    int l2_promotion_;
    int float_oob_fill_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull TmaDescriptorProto_class_data_;
// -------------------------------------------------------------------

class TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::int64_t, ::google::protobuf::Message,
                             ::google::protobuf::internal::WireFormatLite::TYPE_INT64,
                             ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::int64_t, ::google::protobuf::Message,
                      ::google::protobuf::internal::WireFormatLite::TYPE_INT64,
                      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>;
  TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 0,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PROTOBUF_EXPORT TmaMetadataProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:stream_executor.gpu.TmaMetadataProto) */ {
 public:
  inline TmaMetadataProto() : TmaMetadataProto(nullptr) {}
  ~TmaMetadataProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TmaMetadataProto* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TmaMetadataProto));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR TmaMetadataProto(::google::protobuf::internal::ConstantInitialized);

  inline TmaMetadataProto(const TmaMetadataProto& from) : TmaMetadataProto(nullptr, from) {}
  inline TmaMetadataProto(TmaMetadataProto&& from) noexcept
      : TmaMetadataProto(nullptr, ::std::move(from)) {}
  inline TmaMetadataProto& operator=(const TmaMetadataProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline TmaMetadataProto& operator=(TmaMetadataProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const TmaMetadataProto& default_instance() {
    return *reinterpret_cast<const TmaMetadataProto*>(
        &_TmaMetadataProto_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(TmaMetadataProto& a, TmaMetadataProto& b) { a.Swap(&b); }
  inline void Swap(TmaMetadataProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TmaMetadataProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  TmaMetadataProto* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<TmaMetadataProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const TmaMetadataProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const TmaMetadataProto& from) { TmaMetadataProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TmaMetadataProto* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "stream_executor.gpu.TmaMetadataProto"; }

 protected:
  explicit TmaMetadataProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TmaMetadataProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TmaMetadataProto& from);
  TmaMetadataProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TmaMetadataProto&& from) noexcept
      : TmaMetadataProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kArgIndexToTmaInfoFieldNumber = 1,
  };
  // map<int64, .stream_executor.gpu.TmaDescriptorProto> arg_index_to_tma_info = 1;
  int arg_index_to_tma_info_size() const;
  private:
  int _internal_arg_index_to_tma_info_size() const;

  public:
  void clear_arg_index_to_tma_info() ;
  const ::google::protobuf::Map<::int64_t, ::stream_executor::gpu::TmaDescriptorProto>& arg_index_to_tma_info() const;
  ::google::protobuf::Map<::int64_t, ::stream_executor::gpu::TmaDescriptorProto>* PROTOBUF_NONNULL mutable_arg_index_to_tma_info();

  private:
  const ::google::protobuf::Map<::int64_t, ::stream_executor::gpu::TmaDescriptorProto>& _internal_arg_index_to_tma_info() const;
  ::google::protobuf::Map<::int64_t, ::stream_executor::gpu::TmaDescriptorProto>* PROTOBUF_NONNULL _internal_mutable_arg_index_to_tma_info();

  public:
  // @@protoc_insertion_point(class_scope:stream_executor.gpu.TmaMetadataProto)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TmaMetadataProto& from_msg);
    ::google::protobuf::internal::MapField<TmaMetadataProto_ArgIndexToTmaInfoEntry_DoNotUse, ::int64_t, ::stream_executor::gpu::TmaDescriptorProto,
                      ::google::protobuf::internal::WireFormatLite::TYPE_INT64,
                      ::google::protobuf::internal::WireFormatLite::TYPE_MESSAGE>
        arg_index_to_tma_info_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull TmaMetadataProto_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// TmaDescriptorProto

// int64 element_size = 1;
inline void TmaDescriptorProto::clear_element_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.element_size_ = ::int64_t{0};
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline ::int64_t TmaDescriptorProto::element_size() const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.element_size)
  return _internal_element_size();
}
inline void TmaDescriptorProto::set_element_size(::int64_t value) {
  _internal_set_element_size(value);
  _impl_._has_bits_[0] |= 0x00000001u;
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.element_size)
}
inline ::int64_t TmaDescriptorProto::_internal_element_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.element_size_;
}
inline void TmaDescriptorProto::_internal_set_element_size(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.element_size_ = value;
}

// repeated int64 global_dims = 2;
inline int TmaDescriptorProto::_internal_global_dims_size() const {
  return _internal_global_dims().size();
}
inline int TmaDescriptorProto::global_dims_size() const {
  return _internal_global_dims_size();
}
inline void TmaDescriptorProto::clear_global_dims() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_dims_.Clear();
}
inline ::int64_t TmaDescriptorProto::global_dims(int index) const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.global_dims)
  return _internal_global_dims().Get(index);
}
inline void TmaDescriptorProto::set_global_dims(int index, ::int64_t value) {
  _internal_mutable_global_dims()->Set(index, value);
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.global_dims)
}
inline void TmaDescriptorProto::add_global_dims(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_global_dims()->Add(value);
  // @@protoc_insertion_point(field_add:stream_executor.gpu.TmaDescriptorProto.global_dims)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& TmaDescriptorProto::global_dims() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:stream_executor.gpu.TmaDescriptorProto.global_dims)
  return _internal_global_dims();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL TmaDescriptorProto::mutable_global_dims()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:stream_executor.gpu.TmaDescriptorProto.global_dims)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_global_dims();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
TmaDescriptorProto::_internal_global_dims() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.global_dims_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
TmaDescriptorProto::_internal_mutable_global_dims() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.global_dims_;
}

// repeated int64 global_strides = 3;
inline int TmaDescriptorProto::_internal_global_strides_size() const {
  return _internal_global_strides().size();
}
inline int TmaDescriptorProto::global_strides_size() const {
  return _internal_global_strides_size();
}
inline void TmaDescriptorProto::clear_global_strides() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_strides_.Clear();
}
inline ::int64_t TmaDescriptorProto::global_strides(int index) const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.global_strides)
  return _internal_global_strides().Get(index);
}
inline void TmaDescriptorProto::set_global_strides(int index, ::int64_t value) {
  _internal_mutable_global_strides()->Set(index, value);
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.global_strides)
}
inline void TmaDescriptorProto::add_global_strides(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_global_strides()->Add(value);
  // @@protoc_insertion_point(field_add:stream_executor.gpu.TmaDescriptorProto.global_strides)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& TmaDescriptorProto::global_strides() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:stream_executor.gpu.TmaDescriptorProto.global_strides)
  return _internal_global_strides();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL TmaDescriptorProto::mutable_global_strides()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:stream_executor.gpu.TmaDescriptorProto.global_strides)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_global_strides();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
TmaDescriptorProto::_internal_global_strides() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.global_strides_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
TmaDescriptorProto::_internal_mutable_global_strides() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.global_strides_;
}

// repeated int32 box_dims = 4;
inline int TmaDescriptorProto::_internal_box_dims_size() const {
  return _internal_box_dims().size();
}
inline int TmaDescriptorProto::box_dims_size() const {
  return _internal_box_dims_size();
}
inline void TmaDescriptorProto::clear_box_dims() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.box_dims_.Clear();
}
inline ::int32_t TmaDescriptorProto::box_dims(int index) const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.box_dims)
  return _internal_box_dims().Get(index);
}
inline void TmaDescriptorProto::set_box_dims(int index, ::int32_t value) {
  _internal_mutable_box_dims()->Set(index, value);
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.box_dims)
}
inline void TmaDescriptorProto::add_box_dims(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_box_dims()->Add(value);
  // @@protoc_insertion_point(field_add:stream_executor.gpu.TmaDescriptorProto.box_dims)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& TmaDescriptorProto::box_dims() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:stream_executor.gpu.TmaDescriptorProto.box_dims)
  return _internal_box_dims();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL TmaDescriptorProto::mutable_box_dims()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:stream_executor.gpu.TmaDescriptorProto.box_dims)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_box_dims();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
TmaDescriptorProto::_internal_box_dims() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.box_dims_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
TmaDescriptorProto::_internal_mutable_box_dims() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.box_dims_;
}

// repeated int32 element_strides = 5;
inline int TmaDescriptorProto::_internal_element_strides_size() const {
  return _internal_element_strides().size();
}
inline int TmaDescriptorProto::element_strides_size() const {
  return _internal_element_strides_size();
}
inline void TmaDescriptorProto::clear_element_strides() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.element_strides_.Clear();
}
inline ::int32_t TmaDescriptorProto::element_strides(int index) const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.element_strides)
  return _internal_element_strides().Get(index);
}
inline void TmaDescriptorProto::set_element_strides(int index, ::int32_t value) {
  _internal_mutable_element_strides()->Set(index, value);
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.element_strides)
}
inline void TmaDescriptorProto::add_element_strides(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_element_strides()->Add(value);
  // @@protoc_insertion_point(field_add:stream_executor.gpu.TmaDescriptorProto.element_strides)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& TmaDescriptorProto::element_strides() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:stream_executor.gpu.TmaDescriptorProto.element_strides)
  return _internal_element_strides();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL TmaDescriptorProto::mutable_element_strides()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:stream_executor.gpu.TmaDescriptorProto.element_strides)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_element_strides();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
TmaDescriptorProto::_internal_element_strides() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.element_strides_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
TmaDescriptorProto::_internal_mutable_element_strides() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.element_strides_;
}

// .stream_executor.gpu.TmaDescriptorProto.TmaInterleave interleave = 6;
inline void TmaDescriptorProto::clear_interleave() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.interleave_ = 0;
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline ::stream_executor::gpu::TmaDescriptorProto_TmaInterleave TmaDescriptorProto::interleave() const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.interleave)
  return _internal_interleave();
}
inline void TmaDescriptorProto::set_interleave(::stream_executor::gpu::TmaDescriptorProto_TmaInterleave value) {
  _internal_set_interleave(value);
  _impl_._has_bits_[0] |= 0x00000002u;
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.interleave)
}
inline ::stream_executor::gpu::TmaDescriptorProto_TmaInterleave TmaDescriptorProto::_internal_interleave() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::stream_executor::gpu::TmaDescriptorProto_TmaInterleave>(_impl_.interleave_);
}
inline void TmaDescriptorProto::_internal_set_interleave(::stream_executor::gpu::TmaDescriptorProto_TmaInterleave value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.interleave_ = value;
}

// .stream_executor.gpu.TmaDescriptorProto.TmaSwizzle swizzle = 7;
inline void TmaDescriptorProto::clear_swizzle() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.swizzle_ = 0;
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline ::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle TmaDescriptorProto::swizzle() const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.swizzle)
  return _internal_swizzle();
}
inline void TmaDescriptorProto::set_swizzle(::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle value) {
  _internal_set_swizzle(value);
  _impl_._has_bits_[0] |= 0x00000004u;
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.swizzle)
}
inline ::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle TmaDescriptorProto::_internal_swizzle() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle>(_impl_.swizzle_);
}
inline void TmaDescriptorProto::_internal_set_swizzle(::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.swizzle_ = value;
}

// .stream_executor.gpu.TmaDescriptorProto.TmaL2Promotion l2_promotion = 8;
inline void TmaDescriptorProto::clear_l2_promotion() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.l2_promotion_ = 0;
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline ::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion TmaDescriptorProto::l2_promotion() const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.l2_promotion)
  return _internal_l2_promotion();
}
inline void TmaDescriptorProto::set_l2_promotion(::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion value) {
  _internal_set_l2_promotion(value);
  _impl_._has_bits_[0] |= 0x00000008u;
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.l2_promotion)
}
inline ::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion TmaDescriptorProto::_internal_l2_promotion() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion>(_impl_.l2_promotion_);
}
inline void TmaDescriptorProto::_internal_set_l2_promotion(::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.l2_promotion_ = value;
}

// .stream_executor.gpu.TmaDescriptorProto.TmaFloatOobFill float_oob_fill = 9;
inline void TmaDescriptorProto::clear_float_oob_fill() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.float_oob_fill_ = 0;
  _impl_._has_bits_[0] &= ~0x00000010u;
}
inline ::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill TmaDescriptorProto::float_oob_fill() const {
  // @@protoc_insertion_point(field_get:stream_executor.gpu.TmaDescriptorProto.float_oob_fill)
  return _internal_float_oob_fill();
}
inline void TmaDescriptorProto::set_float_oob_fill(::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill value) {
  _internal_set_float_oob_fill(value);
  _impl_._has_bits_[0] |= 0x00000010u;
  // @@protoc_insertion_point(field_set:stream_executor.gpu.TmaDescriptorProto.float_oob_fill)
}
inline ::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill TmaDescriptorProto::_internal_float_oob_fill() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill>(_impl_.float_oob_fill_);
}
inline void TmaDescriptorProto::_internal_set_float_oob_fill(::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.float_oob_fill_ = value;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// TmaMetadataProto

// map<int64, .stream_executor.gpu.TmaDescriptorProto> arg_index_to_tma_info = 1;
inline int TmaMetadataProto::_internal_arg_index_to_tma_info_size() const {
  return _internal_arg_index_to_tma_info().size();
}
inline int TmaMetadataProto::arg_index_to_tma_info_size() const {
  return _internal_arg_index_to_tma_info_size();
}
inline void TmaMetadataProto::clear_arg_index_to_tma_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg_index_to_tma_info_.Clear();
}
inline const ::google::protobuf::Map<::int64_t, ::stream_executor::gpu::TmaDescriptorProto>& TmaMetadataProto::_internal_arg_index_to_tma_info() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arg_index_to_tma_info_.GetMap();
}
inline const ::google::protobuf::Map<::int64_t, ::stream_executor::gpu::TmaDescriptorProto>& TmaMetadataProto::arg_index_to_tma_info() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:stream_executor.gpu.TmaMetadataProto.arg_index_to_tma_info)
  return _internal_arg_index_to_tma_info();
}
inline ::google::protobuf::Map<::int64_t, ::stream_executor::gpu::TmaDescriptorProto>* PROTOBUF_NONNULL TmaMetadataProto::_internal_mutable_arg_index_to_tma_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.arg_index_to_tma_info_.MutableMap();
}
inline ::google::protobuf::Map<::int64_t, ::stream_executor::gpu::TmaDescriptorProto>* PROTOBUF_NONNULL TmaMetadataProto::mutable_arg_index_to_tma_info()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_map:stream_executor.gpu.TmaMetadataProto.arg_index_to_tma_info)
  return _internal_mutable_arg_index_to_tma_info();
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace gpu
}  // namespace stream_executor


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::stream_executor::gpu::TmaDescriptorProto_TmaInterleave> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::stream_executor::gpu::TmaDescriptorProto_TmaInterleave>() {
  return ::stream_executor::gpu::TmaDescriptorProto_TmaInterleave_descriptor();
}
template <>
struct is_proto_enum<::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle>() {
  return ::stream_executor::gpu::TmaDescriptorProto_TmaSwizzle_descriptor();
}
template <>
struct is_proto_enum<::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion>() {
  return ::stream_executor::gpu::TmaDescriptorProto_TmaL2Promotion_descriptor();
}
template <>
struct is_proto_enum<::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill>() {
  return ::stream_executor::gpu::TmaDescriptorProto_TmaFloatOobFill_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // xla_2fstream_5fexecutor_2fgpu_2ftma_5fmetadata_2eproto_2epb_2eh
