// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: tensorflow/core/util/saved_tensor_slice.proto
// Protobuf C++ Version: 6.31.1

#ifndef tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto_2epb_2eh
#define tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6031001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "tensorflow/core/framework/tensor_shape.pb.h"
#include "tensorflow/core/framework/tensor_slice.pb.h"
#include "tensorflow/core/framework/tensor.pb.h"
#include "tensorflow/core/framework/types.pb.h"
#include "tensorflow/core/framework/versions.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto PROTOBUF_EXPORT

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOBUF_EXPORT TableStruct_tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
PROTOBUF_EXPORT extern const ::google::protobuf::internal::DescriptorTable descriptor_table_tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto;
}  // extern "C"
namespace tensorflow {
class SavedSlice;
struct SavedSliceDefaultTypeInternal;
PROTOBUF_EXPORT extern SavedSliceDefaultTypeInternal _SavedSlice_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull SavedSlice_class_data_;
class SavedSliceMeta;
struct SavedSliceMetaDefaultTypeInternal;
PROTOBUF_EXPORT extern SavedSliceMetaDefaultTypeInternal _SavedSliceMeta_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull SavedSliceMeta_class_data_;
class SavedTensorSliceMeta;
struct SavedTensorSliceMetaDefaultTypeInternal;
PROTOBUF_EXPORT extern SavedTensorSliceMetaDefaultTypeInternal _SavedTensorSliceMeta_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull SavedTensorSliceMeta_class_data_;
class SavedTensorSlices;
struct SavedTensorSlicesDefaultTypeInternal;
PROTOBUF_EXPORT extern SavedTensorSlicesDefaultTypeInternal _SavedTensorSlices_default_instance_;
PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull SavedTensorSlices_class_data_;
}  // namespace tensorflow
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace tensorflow {

// ===================================================================


// -------------------------------------------------------------------

class PROTOBUF_EXPORT SavedSliceMeta final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:tensorflow.SavedSliceMeta) */ {
 public:
  inline SavedSliceMeta() : SavedSliceMeta(nullptr) {}
  ~SavedSliceMeta() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SavedSliceMeta* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SavedSliceMeta));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SavedSliceMeta(::google::protobuf::internal::ConstantInitialized);

  inline SavedSliceMeta(const SavedSliceMeta& from) : SavedSliceMeta(nullptr, from) {}
  inline SavedSliceMeta(SavedSliceMeta&& from) noexcept
      : SavedSliceMeta(nullptr, ::std::move(from)) {}
  inline SavedSliceMeta& operator=(const SavedSliceMeta& from) {
    CopyFrom(from);
    return *this;
  }
  inline SavedSliceMeta& operator=(SavedSliceMeta&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SavedSliceMeta& default_instance() {
    return *reinterpret_cast<const SavedSliceMeta*>(
        &_SavedSliceMeta_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(SavedSliceMeta& a, SavedSliceMeta& b) { a.Swap(&b); }
  inline void Swap(SavedSliceMeta* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SavedSliceMeta* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SavedSliceMeta* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SavedSliceMeta>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SavedSliceMeta& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SavedSliceMeta& from) { SavedSliceMeta::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SavedSliceMeta* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "tensorflow.SavedSliceMeta"; }

 protected:
  explicit SavedSliceMeta(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SavedSliceMeta(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SavedSliceMeta& from);
  SavedSliceMeta(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SavedSliceMeta&& from) noexcept
      : SavedSliceMeta(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSliceFieldNumber = 4,
    kNameFieldNumber = 1,
    kShapeFieldNumber = 2,
    kTypeFieldNumber = 3,
  };
  // repeated .tensorflow.TensorSliceProto slice = 4;
  int slice_size() const;
  private:
  int _internal_slice_size() const;

  public:
  void clear_slice() ;
  ::tensorflow::TensorSliceProto* PROTOBUF_NONNULL mutable_slice(int index);
  ::google::protobuf::RepeatedPtrField<::tensorflow::TensorSliceProto>* PROTOBUF_NONNULL mutable_slice();

  private:
  const ::google::protobuf::RepeatedPtrField<::tensorflow::TensorSliceProto>& _internal_slice() const;
  ::google::protobuf::RepeatedPtrField<::tensorflow::TensorSliceProto>* PROTOBUF_NONNULL _internal_mutable_slice();
  public:
  const ::tensorflow::TensorSliceProto& slice(int index) const;
  ::tensorflow::TensorSliceProto* PROTOBUF_NONNULL add_slice();
  const ::google::protobuf::RepeatedPtrField<::tensorflow::TensorSliceProto>& slice() const;
  // string name = 1;
  void clear_name() ;
  const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // .tensorflow.TensorShapeProto shape = 2;
  bool has_shape() const;
  void clear_shape() ;
  const ::tensorflow::TensorShapeProto& shape() const;
  [[nodiscard]] ::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE release_shape();
  ::tensorflow::TensorShapeProto* PROTOBUF_NONNULL mutable_shape();
  void set_allocated_shape(::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_shape(::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE value);
  ::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE unsafe_arena_release_shape();

  private:
  const ::tensorflow::TensorShapeProto& _internal_shape() const;
  ::tensorflow::TensorShapeProto* PROTOBUF_NONNULL _internal_mutable_shape();

  public:
  // .tensorflow.DataType type = 3;
  void clear_type() ;
  ::tensorflow::DataType type() const;
  void set_type(::tensorflow::DataType value);

  private:
  ::tensorflow::DataType _internal_type() const;
  void _internal_set_type(::tensorflow::DataType value);

  public:
  // @@protoc_insertion_point(class_scope:tensorflow.SavedSliceMeta)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   2, 38,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SavedSliceMeta& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::tensorflow::TensorSliceProto > slice_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE shape_;
    int type_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull SavedSliceMeta_class_data_;
// -------------------------------------------------------------------

class PROTOBUF_EXPORT SavedTensorSliceMeta final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:tensorflow.SavedTensorSliceMeta) */ {
 public:
  inline SavedTensorSliceMeta() : SavedTensorSliceMeta(nullptr) {}
  ~SavedTensorSliceMeta() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SavedTensorSliceMeta* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SavedTensorSliceMeta));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SavedTensorSliceMeta(::google::protobuf::internal::ConstantInitialized);

  inline SavedTensorSliceMeta(const SavedTensorSliceMeta& from) : SavedTensorSliceMeta(nullptr, from) {}
  inline SavedTensorSliceMeta(SavedTensorSliceMeta&& from) noexcept
      : SavedTensorSliceMeta(nullptr, ::std::move(from)) {}
  inline SavedTensorSliceMeta& operator=(const SavedTensorSliceMeta& from) {
    CopyFrom(from);
    return *this;
  }
  inline SavedTensorSliceMeta& operator=(SavedTensorSliceMeta&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SavedTensorSliceMeta& default_instance() {
    return *reinterpret_cast<const SavedTensorSliceMeta*>(
        &_SavedTensorSliceMeta_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(SavedTensorSliceMeta& a, SavedTensorSliceMeta& b) { a.Swap(&b); }
  inline void Swap(SavedTensorSliceMeta* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SavedTensorSliceMeta* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SavedTensorSliceMeta* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SavedTensorSliceMeta>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SavedTensorSliceMeta& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SavedTensorSliceMeta& from) { SavedTensorSliceMeta::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SavedTensorSliceMeta* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "tensorflow.SavedTensorSliceMeta"; }

 protected:
  explicit SavedTensorSliceMeta(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SavedTensorSliceMeta(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SavedTensorSliceMeta& from);
  SavedTensorSliceMeta(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SavedTensorSliceMeta&& from) noexcept
      : SavedTensorSliceMeta(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kTensorFieldNumber = 1,
    kVersionsFieldNumber = 2,
  };
  // repeated .tensorflow.SavedSliceMeta tensor = 1;
  int tensor_size() const;
  private:
  int _internal_tensor_size() const;

  public:
  void clear_tensor() ;
  ::tensorflow::SavedSliceMeta* PROTOBUF_NONNULL mutable_tensor(int index);
  ::google::protobuf::RepeatedPtrField<::tensorflow::SavedSliceMeta>* PROTOBUF_NONNULL mutable_tensor();

  private:
  const ::google::protobuf::RepeatedPtrField<::tensorflow::SavedSliceMeta>& _internal_tensor() const;
  ::google::protobuf::RepeatedPtrField<::tensorflow::SavedSliceMeta>* PROTOBUF_NONNULL _internal_mutable_tensor();
  public:
  const ::tensorflow::SavedSliceMeta& tensor(int index) const;
  ::tensorflow::SavedSliceMeta* PROTOBUF_NONNULL add_tensor();
  const ::google::protobuf::RepeatedPtrField<::tensorflow::SavedSliceMeta>& tensor() const;
  // .tensorflow.VersionDef versions = 2;
  bool has_versions() const;
  void clear_versions() ;
  const ::tensorflow::VersionDef& versions() const;
  [[nodiscard]] ::tensorflow::VersionDef* PROTOBUF_NULLABLE release_versions();
  ::tensorflow::VersionDef* PROTOBUF_NONNULL mutable_versions();
  void set_allocated_versions(::tensorflow::VersionDef* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_versions(::tensorflow::VersionDef* PROTOBUF_NULLABLE value);
  ::tensorflow::VersionDef* PROTOBUF_NULLABLE unsafe_arena_release_versions();

  private:
  const ::tensorflow::VersionDef& _internal_versions() const;
  ::tensorflow::VersionDef* PROTOBUF_NONNULL _internal_mutable_versions();

  public:
  // @@protoc_insertion_point(class_scope:tensorflow.SavedTensorSliceMeta)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SavedTensorSliceMeta& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::tensorflow::SavedSliceMeta > tensor_;
    ::tensorflow::VersionDef* PROTOBUF_NULLABLE versions_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull SavedTensorSliceMeta_class_data_;
// -------------------------------------------------------------------

class PROTOBUF_EXPORT SavedSlice final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:tensorflow.SavedSlice) */ {
 public:
  inline SavedSlice() : SavedSlice(nullptr) {}
  ~SavedSlice() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SavedSlice* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SavedSlice));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SavedSlice(::google::protobuf::internal::ConstantInitialized);

  inline SavedSlice(const SavedSlice& from) : SavedSlice(nullptr, from) {}
  inline SavedSlice(SavedSlice&& from) noexcept
      : SavedSlice(nullptr, ::std::move(from)) {}
  inline SavedSlice& operator=(const SavedSlice& from) {
    CopyFrom(from);
    return *this;
  }
  inline SavedSlice& operator=(SavedSlice&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SavedSlice& default_instance() {
    return *reinterpret_cast<const SavedSlice*>(
        &_SavedSlice_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(SavedSlice& a, SavedSlice& b) { a.Swap(&b); }
  inline void Swap(SavedSlice* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SavedSlice* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SavedSlice* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SavedSlice>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SavedSlice& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SavedSlice& from) { SavedSlice::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SavedSlice* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "tensorflow.SavedSlice"; }

 protected:
  explicit SavedSlice(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SavedSlice(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SavedSlice& from);
  SavedSlice(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SavedSlice&& from) noexcept
      : SavedSlice(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kSliceFieldNumber = 2,
    kDataFieldNumber = 3,
  };
  // string name = 1;
  void clear_name() ;
  const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // .tensorflow.TensorSliceProto slice = 2;
  bool has_slice() const;
  void clear_slice() ;
  const ::tensorflow::TensorSliceProto& slice() const;
  [[nodiscard]] ::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE release_slice();
  ::tensorflow::TensorSliceProto* PROTOBUF_NONNULL mutable_slice();
  void set_allocated_slice(::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_slice(::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE value);
  ::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE unsafe_arena_release_slice();

  private:
  const ::tensorflow::TensorSliceProto& _internal_slice() const;
  ::tensorflow::TensorSliceProto* PROTOBUF_NONNULL _internal_mutable_slice();

  public:
  // .tensorflow.TensorProto data = 3;
  bool has_data() const;
  void clear_data() ;
  const ::tensorflow::TensorProto& data() const;
  [[nodiscard]] ::tensorflow::TensorProto* PROTOBUF_NULLABLE release_data();
  ::tensorflow::TensorProto* PROTOBUF_NONNULL mutable_data();
  void set_allocated_data(::tensorflow::TensorProto* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_data(::tensorflow::TensorProto* PROTOBUF_NULLABLE value);
  ::tensorflow::TensorProto* PROTOBUF_NULLABLE unsafe_arena_release_data();

  private:
  const ::tensorflow::TensorProto& _internal_data() const;
  ::tensorflow::TensorProto* PROTOBUF_NONNULL _internal_mutable_data();

  public:
  // @@protoc_insertion_point(class_scope:tensorflow.SavedSlice)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   2, 34,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SavedSlice& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE slice_;
    ::tensorflow::TensorProto* PROTOBUF_NULLABLE data_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull SavedSlice_class_data_;
// -------------------------------------------------------------------

class PROTOBUF_EXPORT SavedTensorSlices final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:tensorflow.SavedTensorSlices) */ {
 public:
  inline SavedTensorSlices() : SavedTensorSlices(nullptr) {}
  ~SavedTensorSlices() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SavedTensorSlices* PROTOBUF_NONNULL msg, std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SavedTensorSlices));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SavedTensorSlices(::google::protobuf::internal::ConstantInitialized);

  inline SavedTensorSlices(const SavedTensorSlices& from) : SavedTensorSlices(nullptr, from) {}
  inline SavedTensorSlices(SavedTensorSlices&& from) noexcept
      : SavedTensorSlices(nullptr, ::std::move(from)) {}
  inline SavedTensorSlices& operator=(const SavedTensorSlices& from) {
    CopyFrom(from);
    return *this;
  }
  inline SavedTensorSlices& operator=(SavedTensorSlices&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SavedTensorSlices& default_instance() {
    return *reinterpret_cast<const SavedTensorSlices*>(
        &_SavedTensorSlices_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(SavedTensorSlices& a, SavedTensorSlices& b) { a.Swap(&b); }
  inline void Swap(SavedTensorSlices* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SavedTensorSlices* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SavedTensorSlices* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SavedTensorSlices>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SavedTensorSlices& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SavedTensorSlices& from) { SavedTensorSlices::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SavedTensorSlices* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "tensorflow.SavedTensorSlices"; }

 protected:
  explicit SavedTensorSlices(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SavedTensorSlices(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SavedTensorSlices& from);
  SavedTensorSlices(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SavedTensorSlices&& from) noexcept
      : SavedTensorSlices(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kMetaFieldNumber = 1,
    kDataFieldNumber = 2,
  };
  // .tensorflow.SavedTensorSliceMeta meta = 1;
  bool has_meta() const;
  void clear_meta() ;
  const ::tensorflow::SavedTensorSliceMeta& meta() const;
  [[nodiscard]] ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE release_meta();
  ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NONNULL mutable_meta();
  void set_allocated_meta(::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_meta(::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE value);
  ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE unsafe_arena_release_meta();

  private:
  const ::tensorflow::SavedTensorSliceMeta& _internal_meta() const;
  ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NONNULL _internal_mutable_meta();

  public:
  // .tensorflow.SavedSlice data = 2;
  bool has_data() const;
  void clear_data() ;
  const ::tensorflow::SavedSlice& data() const;
  [[nodiscard]] ::tensorflow::SavedSlice* PROTOBUF_NULLABLE release_data();
  ::tensorflow::SavedSlice* PROTOBUF_NONNULL mutable_data();
  void set_allocated_data(::tensorflow::SavedSlice* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_data(::tensorflow::SavedSlice* PROTOBUF_NULLABLE value);
  ::tensorflow::SavedSlice* PROTOBUF_NULLABLE unsafe_arena_release_data();

  private:
  const ::tensorflow::SavedSlice& _internal_data() const;
  ::tensorflow::SavedSlice* PROTOBUF_NONNULL _internal_mutable_data();

  public:
  // @@protoc_insertion_point(class_scope:tensorflow.SavedTensorSlices)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SavedTensorSlices& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE meta_;
    ::tensorflow::SavedSlice* PROTOBUF_NULLABLE data_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto;
};

PROTOBUF_EXPORT extern const ::google::protobuf::internal::ClassDataFull SavedTensorSlices_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// SavedSliceMeta

// string name = 1;
inline void SavedSliceMeta::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::std::string& SavedSliceMeta::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedSliceMeta.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SavedSliceMeta::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:tensorflow.SavedSliceMeta.name)
}
inline ::std::string* PROTOBUF_NONNULL SavedSliceMeta::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedSliceMeta.name)
  return _s;
}
inline const ::std::string& SavedSliceMeta::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void SavedSliceMeta::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SavedSliceMeta::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SavedSliceMeta::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.SavedSliceMeta.name)
  if ((_impl_._has_bits_[0] & 0x00000001u) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void SavedSliceMeta::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:tensorflow.SavedSliceMeta.name)
}

// .tensorflow.TensorShapeProto shape = 2;
inline bool SavedSliceMeta::has_shape() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.shape_ != nullptr);
  return value;
}
inline const ::tensorflow::TensorShapeProto& SavedSliceMeta::_internal_shape() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::tensorflow::TensorShapeProto* p = _impl_.shape_;
  return p != nullptr ? *p : reinterpret_cast<const ::tensorflow::TensorShapeProto&>(::tensorflow::_TensorShapeProto_default_instance_);
}
inline const ::tensorflow::TensorShapeProto& SavedSliceMeta::shape() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedSliceMeta.shape)
  return _internal_shape();
}
inline void SavedSliceMeta::unsafe_arena_set_allocated_shape(
    ::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.shape_);
  }
  _impl_.shape_ = reinterpret_cast<::tensorflow::TensorShapeProto*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:tensorflow.SavedSliceMeta.shape)
}
inline ::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE SavedSliceMeta::release_shape() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000002u;
  ::tensorflow::TensorShapeProto* released = _impl_.shape_;
  _impl_.shape_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE SavedSliceMeta::unsafe_arena_release_shape() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.SavedSliceMeta.shape)

  _impl_._has_bits_[0] &= ~0x00000002u;
  ::tensorflow::TensorShapeProto* temp = _impl_.shape_;
  _impl_.shape_ = nullptr;
  return temp;
}
inline ::tensorflow::TensorShapeProto* PROTOBUF_NONNULL SavedSliceMeta::_internal_mutable_shape() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.shape_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::tensorflow::TensorShapeProto>(GetArena());
    _impl_.shape_ = reinterpret_cast<::tensorflow::TensorShapeProto*>(p);
  }
  return _impl_.shape_;
}
inline ::tensorflow::TensorShapeProto* PROTOBUF_NONNULL SavedSliceMeta::mutable_shape()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000002u;
  ::tensorflow::TensorShapeProto* _msg = _internal_mutable_shape();
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedSliceMeta.shape)
  return _msg;
}
inline void SavedSliceMeta::set_allocated_shape(::tensorflow::TensorShapeProto* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.shape_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }

  _impl_.shape_ = reinterpret_cast<::tensorflow::TensorShapeProto*>(value);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.SavedSliceMeta.shape)
}

// .tensorflow.DataType type = 3;
inline void SavedSliceMeta::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline ::tensorflow::DataType SavedSliceMeta::type() const {
  // @@protoc_insertion_point(field_get:tensorflow.SavedSliceMeta.type)
  return _internal_type();
}
inline void SavedSliceMeta::set_type(::tensorflow::DataType value) {
  _internal_set_type(value);
  _impl_._has_bits_[0] |= 0x00000004u;
  // @@protoc_insertion_point(field_set:tensorflow.SavedSliceMeta.type)
}
inline ::tensorflow::DataType SavedSliceMeta::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::tensorflow::DataType>(_impl_.type_);
}
inline void SavedSliceMeta::_internal_set_type(::tensorflow::DataType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = value;
}

// repeated .tensorflow.TensorSliceProto slice = 4;
inline int SavedSliceMeta::_internal_slice_size() const {
  return _internal_slice().size();
}
inline int SavedSliceMeta::slice_size() const {
  return _internal_slice_size();
}
inline ::tensorflow::TensorSliceProto* PROTOBUF_NONNULL SavedSliceMeta::mutable_slice(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedSliceMeta.slice)
  return _internal_mutable_slice()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::tensorflow::TensorSliceProto>* PROTOBUF_NONNULL SavedSliceMeta::mutable_slice()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:tensorflow.SavedSliceMeta.slice)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_slice();
}
inline const ::tensorflow::TensorSliceProto& SavedSliceMeta::slice(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedSliceMeta.slice)
  return _internal_slice().Get(index);
}
inline ::tensorflow::TensorSliceProto* PROTOBUF_NONNULL SavedSliceMeta::add_slice()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::tensorflow::TensorSliceProto* _add = _internal_mutable_slice()->Add();
  // @@protoc_insertion_point(field_add:tensorflow.SavedSliceMeta.slice)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::tensorflow::TensorSliceProto>& SavedSliceMeta::slice() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:tensorflow.SavedSliceMeta.slice)
  return _internal_slice();
}
inline const ::google::protobuf::RepeatedPtrField<::tensorflow::TensorSliceProto>&
SavedSliceMeta::_internal_slice() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.slice_;
}
inline ::google::protobuf::RepeatedPtrField<::tensorflow::TensorSliceProto>* PROTOBUF_NONNULL
SavedSliceMeta::_internal_mutable_slice() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.slice_;
}

// -------------------------------------------------------------------

// SavedTensorSliceMeta

// repeated .tensorflow.SavedSliceMeta tensor = 1;
inline int SavedTensorSliceMeta::_internal_tensor_size() const {
  return _internal_tensor().size();
}
inline int SavedTensorSliceMeta::tensor_size() const {
  return _internal_tensor_size();
}
inline void SavedTensorSliceMeta::clear_tensor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tensor_.Clear();
}
inline ::tensorflow::SavedSliceMeta* PROTOBUF_NONNULL SavedTensorSliceMeta::mutable_tensor(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedTensorSliceMeta.tensor)
  return _internal_mutable_tensor()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::tensorflow::SavedSliceMeta>* PROTOBUF_NONNULL SavedTensorSliceMeta::mutable_tensor()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable_list:tensorflow.SavedTensorSliceMeta.tensor)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_tensor();
}
inline const ::tensorflow::SavedSliceMeta& SavedTensorSliceMeta::tensor(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedTensorSliceMeta.tensor)
  return _internal_tensor().Get(index);
}
inline ::tensorflow::SavedSliceMeta* PROTOBUF_NONNULL SavedTensorSliceMeta::add_tensor()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::tensorflow::SavedSliceMeta* _add = _internal_mutable_tensor()->Add();
  // @@protoc_insertion_point(field_add:tensorflow.SavedTensorSliceMeta.tensor)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::tensorflow::SavedSliceMeta>& SavedTensorSliceMeta::tensor() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:tensorflow.SavedTensorSliceMeta.tensor)
  return _internal_tensor();
}
inline const ::google::protobuf::RepeatedPtrField<::tensorflow::SavedSliceMeta>&
SavedTensorSliceMeta::_internal_tensor() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.tensor_;
}
inline ::google::protobuf::RepeatedPtrField<::tensorflow::SavedSliceMeta>* PROTOBUF_NONNULL
SavedTensorSliceMeta::_internal_mutable_tensor() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.tensor_;
}

// .tensorflow.VersionDef versions = 2;
inline bool SavedTensorSliceMeta::has_versions() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.versions_ != nullptr);
  return value;
}
inline const ::tensorflow::VersionDef& SavedTensorSliceMeta::_internal_versions() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::tensorflow::VersionDef* p = _impl_.versions_;
  return p != nullptr ? *p : reinterpret_cast<const ::tensorflow::VersionDef&>(::tensorflow::_VersionDef_default_instance_);
}
inline const ::tensorflow::VersionDef& SavedTensorSliceMeta::versions() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedTensorSliceMeta.versions)
  return _internal_versions();
}
inline void SavedTensorSliceMeta::unsafe_arena_set_allocated_versions(
    ::tensorflow::VersionDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.versions_);
  }
  _impl_.versions_ = reinterpret_cast<::tensorflow::VersionDef*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:tensorflow.SavedTensorSliceMeta.versions)
}
inline ::tensorflow::VersionDef* PROTOBUF_NULLABLE SavedTensorSliceMeta::release_versions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000001u;
  ::tensorflow::VersionDef* released = _impl_.versions_;
  _impl_.versions_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::tensorflow::VersionDef* PROTOBUF_NULLABLE SavedTensorSliceMeta::unsafe_arena_release_versions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.SavedTensorSliceMeta.versions)

  _impl_._has_bits_[0] &= ~0x00000001u;
  ::tensorflow::VersionDef* temp = _impl_.versions_;
  _impl_.versions_ = nullptr;
  return temp;
}
inline ::tensorflow::VersionDef* PROTOBUF_NONNULL SavedTensorSliceMeta::_internal_mutable_versions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.versions_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::tensorflow::VersionDef>(GetArena());
    _impl_.versions_ = reinterpret_cast<::tensorflow::VersionDef*>(p);
  }
  return _impl_.versions_;
}
inline ::tensorflow::VersionDef* PROTOBUF_NONNULL SavedTensorSliceMeta::mutable_versions()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000001u;
  ::tensorflow::VersionDef* _msg = _internal_mutable_versions();
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedTensorSliceMeta.versions)
  return _msg;
}
inline void SavedTensorSliceMeta::set_allocated_versions(::tensorflow::VersionDef* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.versions_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }

  _impl_.versions_ = reinterpret_cast<::tensorflow::VersionDef*>(value);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.SavedTensorSliceMeta.versions)
}

// -------------------------------------------------------------------

// SavedSlice

// string name = 1;
inline void SavedSlice::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::std::string& SavedSlice::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedSlice.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SavedSlice::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:tensorflow.SavedSlice.name)
}
inline ::std::string* PROTOBUF_NONNULL SavedSlice::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedSlice.name)
  return _s;
}
inline const ::std::string& SavedSlice::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void SavedSlice::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SavedSlice::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SavedSlice::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.SavedSlice.name)
  if ((_impl_._has_bits_[0] & 0x00000001u) == 0) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void SavedSlice::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:tensorflow.SavedSlice.name)
}

// .tensorflow.TensorSliceProto slice = 2;
inline bool SavedSlice::has_slice() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.slice_ != nullptr);
  return value;
}
inline const ::tensorflow::TensorSliceProto& SavedSlice::_internal_slice() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::tensorflow::TensorSliceProto* p = _impl_.slice_;
  return p != nullptr ? *p : reinterpret_cast<const ::tensorflow::TensorSliceProto&>(::tensorflow::_TensorSliceProto_default_instance_);
}
inline const ::tensorflow::TensorSliceProto& SavedSlice::slice() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedSlice.slice)
  return _internal_slice();
}
inline void SavedSlice::unsafe_arena_set_allocated_slice(
    ::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.slice_);
  }
  _impl_.slice_ = reinterpret_cast<::tensorflow::TensorSliceProto*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:tensorflow.SavedSlice.slice)
}
inline ::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE SavedSlice::release_slice() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000002u;
  ::tensorflow::TensorSliceProto* released = _impl_.slice_;
  _impl_.slice_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE SavedSlice::unsafe_arena_release_slice() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.SavedSlice.slice)

  _impl_._has_bits_[0] &= ~0x00000002u;
  ::tensorflow::TensorSliceProto* temp = _impl_.slice_;
  _impl_.slice_ = nullptr;
  return temp;
}
inline ::tensorflow::TensorSliceProto* PROTOBUF_NONNULL SavedSlice::_internal_mutable_slice() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.slice_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::tensorflow::TensorSliceProto>(GetArena());
    _impl_.slice_ = reinterpret_cast<::tensorflow::TensorSliceProto*>(p);
  }
  return _impl_.slice_;
}
inline ::tensorflow::TensorSliceProto* PROTOBUF_NONNULL SavedSlice::mutable_slice()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000002u;
  ::tensorflow::TensorSliceProto* _msg = _internal_mutable_slice();
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedSlice.slice)
  return _msg;
}
inline void SavedSlice::set_allocated_slice(::tensorflow::TensorSliceProto* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.slice_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }

  _impl_.slice_ = reinterpret_cast<::tensorflow::TensorSliceProto*>(value);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.SavedSlice.slice)
}

// .tensorflow.TensorProto data = 3;
inline bool SavedSlice::has_data() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.data_ != nullptr);
  return value;
}
inline const ::tensorflow::TensorProto& SavedSlice::_internal_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::tensorflow::TensorProto* p = _impl_.data_;
  return p != nullptr ? *p : reinterpret_cast<const ::tensorflow::TensorProto&>(::tensorflow::_TensorProto_default_instance_);
}
inline const ::tensorflow::TensorProto& SavedSlice::data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedSlice.data)
  return _internal_data();
}
inline void SavedSlice::unsafe_arena_set_allocated_data(
    ::tensorflow::TensorProto* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.data_);
  }
  _impl_.data_ = reinterpret_cast<::tensorflow::TensorProto*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:tensorflow.SavedSlice.data)
}
inline ::tensorflow::TensorProto* PROTOBUF_NULLABLE SavedSlice::release_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000004u;
  ::tensorflow::TensorProto* released = _impl_.data_;
  _impl_.data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::tensorflow::TensorProto* PROTOBUF_NULLABLE SavedSlice::unsafe_arena_release_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.SavedSlice.data)

  _impl_._has_bits_[0] &= ~0x00000004u;
  ::tensorflow::TensorProto* temp = _impl_.data_;
  _impl_.data_ = nullptr;
  return temp;
}
inline ::tensorflow::TensorProto* PROTOBUF_NONNULL SavedSlice::_internal_mutable_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.data_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::tensorflow::TensorProto>(GetArena());
    _impl_.data_ = reinterpret_cast<::tensorflow::TensorProto*>(p);
  }
  return _impl_.data_;
}
inline ::tensorflow::TensorProto* PROTOBUF_NONNULL SavedSlice::mutable_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000004u;
  ::tensorflow::TensorProto* _msg = _internal_mutable_data();
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedSlice.data)
  return _msg;
}
inline void SavedSlice::set_allocated_data(::tensorflow::TensorProto* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }

  _impl_.data_ = reinterpret_cast<::tensorflow::TensorProto*>(value);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.SavedSlice.data)
}

// -------------------------------------------------------------------

// SavedTensorSlices

// .tensorflow.SavedTensorSliceMeta meta = 1;
inline bool SavedTensorSlices::has_meta() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.meta_ != nullptr);
  return value;
}
inline void SavedTensorSlices::clear_meta() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.meta_ != nullptr) _impl_.meta_->Clear();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::tensorflow::SavedTensorSliceMeta& SavedTensorSlices::_internal_meta() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::tensorflow::SavedTensorSliceMeta* p = _impl_.meta_;
  return p != nullptr ? *p : reinterpret_cast<const ::tensorflow::SavedTensorSliceMeta&>(::tensorflow::_SavedTensorSliceMeta_default_instance_);
}
inline const ::tensorflow::SavedTensorSliceMeta& SavedTensorSlices::meta() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedTensorSlices.meta)
  return _internal_meta();
}
inline void SavedTensorSlices::unsafe_arena_set_allocated_meta(
    ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.meta_);
  }
  _impl_.meta_ = reinterpret_cast<::tensorflow::SavedTensorSliceMeta*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:tensorflow.SavedTensorSlices.meta)
}
inline ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE SavedTensorSlices::release_meta() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000001u;
  ::tensorflow::SavedTensorSliceMeta* released = _impl_.meta_;
  _impl_.meta_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE SavedTensorSlices::unsafe_arena_release_meta() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.SavedTensorSlices.meta)

  _impl_._has_bits_[0] &= ~0x00000001u;
  ::tensorflow::SavedTensorSliceMeta* temp = _impl_.meta_;
  _impl_.meta_ = nullptr;
  return temp;
}
inline ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NONNULL SavedTensorSlices::_internal_mutable_meta() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.meta_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::tensorflow::SavedTensorSliceMeta>(GetArena());
    _impl_.meta_ = reinterpret_cast<::tensorflow::SavedTensorSliceMeta*>(p);
  }
  return _impl_.meta_;
}
inline ::tensorflow::SavedTensorSliceMeta* PROTOBUF_NONNULL SavedTensorSlices::mutable_meta()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000001u;
  ::tensorflow::SavedTensorSliceMeta* _msg = _internal_mutable_meta();
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedTensorSlices.meta)
  return _msg;
}
inline void SavedTensorSlices::set_allocated_meta(::tensorflow::SavedTensorSliceMeta* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.meta_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }

  _impl_.meta_ = reinterpret_cast<::tensorflow::SavedTensorSliceMeta*>(value);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.SavedTensorSlices.meta)
}

// .tensorflow.SavedSlice data = 2;
inline bool SavedTensorSlices::has_data() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.data_ != nullptr);
  return value;
}
inline void SavedTensorSlices::clear_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.data_ != nullptr) _impl_.data_->Clear();
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline const ::tensorflow::SavedSlice& SavedTensorSlices::_internal_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::tensorflow::SavedSlice* p = _impl_.data_;
  return p != nullptr ? *p : reinterpret_cast<const ::tensorflow::SavedSlice&>(::tensorflow::_SavedSlice_default_instance_);
}
inline const ::tensorflow::SavedSlice& SavedTensorSlices::data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:tensorflow.SavedTensorSlices.data)
  return _internal_data();
}
inline void SavedTensorSlices::unsafe_arena_set_allocated_data(
    ::tensorflow::SavedSlice* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.data_);
  }
  _impl_.data_ = reinterpret_cast<::tensorflow::SavedSlice*>(value);
  if (value != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:tensorflow.SavedTensorSlices.data)
}
inline ::tensorflow::SavedSlice* PROTOBUF_NULLABLE SavedTensorSlices::release_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  _impl_._has_bits_[0] &= ~0x00000002u;
  ::tensorflow::SavedSlice* released = _impl_.data_;
  _impl_.data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::tensorflow::SavedSlice* PROTOBUF_NULLABLE SavedTensorSlices::unsafe_arena_release_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:tensorflow.SavedTensorSlices.data)

  _impl_._has_bits_[0] &= ~0x00000002u;
  ::tensorflow::SavedSlice* temp = _impl_.data_;
  _impl_.data_ = nullptr;
  return temp;
}
inline ::tensorflow::SavedSlice* PROTOBUF_NONNULL SavedTensorSlices::_internal_mutable_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.data_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::tensorflow::SavedSlice>(GetArena());
    _impl_.data_ = reinterpret_cast<::tensorflow::SavedSlice*>(p);
  }
  return _impl_.data_;
}
inline ::tensorflow::SavedSlice* PROTOBUF_NONNULL SavedTensorSlices::mutable_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  _impl_._has_bits_[0] |= 0x00000002u;
  ::tensorflow::SavedSlice* _msg = _internal_mutable_data();
  // @@protoc_insertion_point(field_mutable:tensorflow.SavedTensorSlices.data)
  return _msg;
}
inline void SavedTensorSlices::set_allocated_data(::tensorflow::SavedSlice* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }

  _impl_.data_ = reinterpret_cast<::tensorflow::SavedSlice*>(value);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.SavedTensorSlices.data)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace tensorflow


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // tensorflow_2fcore_2futil_2fsaved_5ftensor_5fslice_2eproto_2epb_2eh
