
/* Autogenerated by mlir-tblgen; don't manually edit. */

#include "mlir-c/Pass.h"

#ifdef __cplusplus
extern "C" {
#endif

// Registration for the entire group
MLIR_CAPI_EXPORTED void mlirRegisterPasses(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateCheckUsesPass(void);
MLIR_CAPI_EXPORTED void mlirRegisterCheckUsesPass(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateInferEffectsPass(void);
MLIR_CAPI_EXPORTED void mlirRegisterInferEffectsPass(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreateInterpreterPass(void);
MLIR_CAPI_EXPORTED void mlirRegisterInterpreterPass(void);


/* Create  Pass. */
MLIR_CAPI_EXPORTED MlirPass mlirCreatePreloadLibraryPass(void);
MLIR_CAPI_EXPORTED void mlirRegisterPreloadLibraryPass(void);



#ifdef __cplusplus
}
#endif
