YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_1x32x2_1x8x2_avx2_fma3, 1, 32, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_2x32x2_1x8x2_avx2_fma3, 2, 32, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_2x16x2_1x8x2_avx2_fma3, 2, 16, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_3x16x2_1x8x2_avx2_fma3, 3, 16, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_4x16x2_1x8x2_avx2_fma3, 4, 16, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_5x16x2_1x8x2_avx2_fma3, 5, 16, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_8x8x2_1x8x2_avx2_fma3, 8, 8, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_10x8x2_1x8x2_avx2_fma3, 10, 8, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
YNN_DOT_KERNEL(arch_flag::avx2_fma3, dot_bf16_bf16_fp32_12x8x2_1x8x2_avx2_fma3, 12, 8, 2, 1, 8, 2, /*flags=*/dot_flag::consistent_arithmetic, bfloat16, bfloat16, float)
